;------------------------------------------------------------------------------
; KC-85-RAM-Forth
; FIG-FORTH basierend, Version 1.2
;------------------------------------------------------------------------------
; File Name   :	kc85 forth\FORTH-KC_0248_20B7.rom
; Base Address:	0000h Range: 0248h - 20C8h Loaded length: 1E80h
;
; assemblieren mit macro assembler as 1.42 Beta [Bld 82] oder neuer !!
;
; reass. V.Pohlers 10.2012, KC85-spezifisches M. Leubner
; letzte nderung: 02.10.2012 15:35
;------------------------------------------------------------------------------

		cpu	z80

;------------------------------------------------------------------------------
;	KC85-Vereinbarungen

PIO_A	equ	88H	; PIO Port A
			; Bit 5 = LED
			; Bit 6 = Kassettenrecorder (Motorschaltspannung)

ARG3		equ	0B786H		; Argument 3
WINON		equ	0B79CH		; Fensteranfang
WINLG		equ	0B79EH		; Fenstergroesse
CURSO		equ	0B7A0H		; Cursorposition
COLR		equ	0B7A3H		; Textfarbe
SUTAB		equ	0B7B0H		; CAOS-Unterprogrammtabelle
HOR		equ	0B7D3H		; X-Wert fuer Grafikroutinen
VERT		equ	0B7D5H		; Y-Wert fuer Grafikroutinen
FARB		equ	0B7D6H		; Grafikfarbe

PV1		equ	0F003H		; CAOS-Programmverteiler I   (DB UP-Nr.)
PV3		equ	0F009H		; CAOS-Programmverteiler III (E=UP-Nr.)
PV4		equ	0F00CH		; CAOS-Programmverteiler IV  (E=UP-Nr. + IRM on/off)
CRT		equ	00H		; Zeichen auf Bildschirm anzeigen
KBD		equ	04H		; Tasteneingabe mit Cursoreinblendung
KBDZ		equ	0EH		; Tastaturabfrage
INTB		equ	16H		; Zeicheneingabe
OCHR		equ	24H		; Zeichenausgabe
LDMA		equ	28H		; Zeichen in Speicher schreiben
LDAM		equ	29H		; Zeichen aus Speicher lesen
PUDE		equ	2FH		; Bildpunkt loeschen
PUSE		equ	30H		; Bildpunkt setzen
SIXD		equ	31H		; Verlagerung IX-Arbeitsbereich
DABR		equ	32h		;
TON		equ	35H		; Tonausgabe

;------------------------------------------------------------------------------

UP		equ	017EH		; USER VAR BASE ADDR


		org	248h

		dw	7F7Fh
aForth:		db	"FORTH",0
		jp	CLD
		dw	7F7Fh
aReforth:	db	"REFORTH",0
		jp	WRM

;------------------------------------------------------------------------------

ORIG:		db	0
		jp	CLD		; VECTOR TO COLD START
		db	0
		jp	WRM		; VECTOR TO WARM START
		dw	1		; FIG RELEASE #
		dw	2		; FIG REVISION #
;<<<<<< FOLLOWING USED BY COLD;
;	MUST BE IN SAME ORDER AS USER VARIABLES
		dw	TASK_		; TOPMOST WORD IN FORTH VOCABULARY
		dw	8		; ASCII-Code fr BACKSPACE
		dw	200h		; Init User-Area-Pointer	
		dw	0E0h		; 6 INIT (S0)
		dw	UP		; 8 INIT (R0)
		dw	0E0h		; 10 INIT (TIB)
		dw	20h		; 12 INIT (WIDTH)
		dw	0		; 14 INIT (WARNING)
		dw	INITDP		; 16 INIT (FENCE)
		dw	INITDP		; 18 INIT (DP)
		dw	off_1DD3	; 20 INIT (VOC-LINK)
		dw	2FF0h		; 22 INIT (FIRST)
		dw	4000h		; 24 INIT (LIMIT)
unk_286:	dw	0		; 26 INIT (IOTAB)
		dw	0		; Initialisierung OFFSET


;------------------------------------------------------------------------------
;	CONSOLE INTERFACE

; console in
CIN:		ld	e,INTB		; 16h
		call	PV4		; Zeicheneingabe
		or	a
		jr	z, CIN		; WAIT FOR CHR TO BE TYPED
		ret			; (A) <- CHR

; console status	
CSTAT:		ld	e,KBDZ		; 0Eh
		call	PV4		; Tastaturabfrage
		jr	c, CSTS1	; CHECK IF ANY CHR HAS BEEN TYPED
		xor	a		; IF NOT CHR TYPED THEN (A) <- 0
		ret
CSTS1:		ld	a, 1		; ELSE (A) <- 1
		ret

; console out
COUT:		and	7Fh		; Bit 7 abschneiden (nicht darstellbar)
		ld	e,OCHR		; 24h
		call	PV4		; Zeichenausgabe (Zeichen in Reg. A)
		ret
;
		db	0
		db	0
		db	0
		db	0
		db	0
		db	0
		db	0
		db	0
		db	0
		db	0
		db	0
		db	0
		db	0
		db	0
		db	0
		db	0
		db	0
		db	0
		db	0
		db	0
		db	0
		db	0
		db	0

;------------------------------------------------------------------------------
; Makro fr FORTH-Header
; Aufruf:	header	EXEC,"EXECUTE"				( primitive )
;		header	COLON,":",DOCOL,IMMEDIATE		( Colon-Definition )

edlnk0		eval	0					; erste LFA

header		MACRO	LABL,NAME, CFA, HBITS, LFA, {NoExpand}
onlnk0		eval	$
		if "HBITS"<>""
			db	80h | StrLen(NAME) | HBITS
		else
			db	80h | StrLen(NAME)
		endif
		if StrLen(NAME)>1
			db	SubStr(NAME,0,StrLen(NAME)-1)
		endif
		db	charfromstr(NAME,StrLen(NAME)-1) | 80h
		; LFA
		if "LFA"<>""
			dw	LFA
		else
			dw	edlnk0
		endif
edlnk0		eval 	onlnk0
		; CFA
LABL		label	$
		if "CFA"<>""
			dw	CFA
		else
			dw	$+2	; primitive
		endif
		ENDM

IMMEDIATE	equ	1<<6		; Immediate-Bit

;------------------------------------------------------
;
;	FORTH REGISTERS
;
;	FORTH	Z80	FORTH PRESERVATION RULES
;	-----	----	---------------------------------------
;			  FORTH WORDS
;	W	DE	SOMETIMES OUTPUT FROM NEXT
;			MAY BE ALTERED BEFORE JMP'ING TO NEXT
;			INPUT ONLY WHEN 'DPUSH' CALLED
;	SP	SP	SHOULD BE USED ONLY AS DATA STACK
;			  ACROSS FORTH WORDS
;			MAY BE USED WITHIN FORTH WORDS
;			  IF RESTORED BEFORE 'NEXT'
;		HL	NEVER OUTPUT FROM NEXT
;			INPUT ONLY WHEN 'HPUSH' CALLED
;	RP	IY	RETURN STACK POINTER
;	 R1	IY+0
;	 R2	IY+2
;
;------------------------------------------------------
;
;	COMMENT CONVENTIONS:
;
;	=	MEANS	"IS EQUAL TO"
;	<-	MEANS	ASSIGNMENT
;
;	NAME	=	ADDRESS OF NAME
;	(NAME)	=	CONTENTS AT NAME
;	((NAME))=	INDIRECT CONTENTS
;
;	CFA	=	ADDRESS OF CODE FIELD
;	LFA	=	ADDRESS OF LINK FIELD
;	NFA	=	ADDR OF START OF NAME FIELD
;	PFA	=	ADDR OF START OF PARAMETER FIELD
;
;	S1	=	ADDR OF 1ST WORD OF PARAMETER STACK
;	S2	=	ADDR OF 2ND WORD OF PARAMETER STACK
;	R1	=	ADDR OF 1ST WORD OF RETURN STACK
;	R2	=	ADDR OF 2ND WORD OF RETURN STACK
;	( ABOVE STACK POSITIONS VALID BEFORE & AFTER EXECUTION
;	OF ANY WORD, NOT DURING. )
;
;	LSB	=	LEAST SIGNIFICANT BIT
;	MSB	=	MOST SIGNIFICANT BIT
;	LB	=	LOW BYTE
;	HB	=	HIGH BYTE
;	LW	=	LOW WORD
;	HW	=	HIGH WORD
;	( MAY BE USED AS SUFFIX TO ABOVE NAMES )
;--------------------------------------------------
;
;	NEXT, THE FORTH ADDRESS INTERPRETER
;	  ( POST INCREMENTING VERSION )
;
DPUSH:		push	de
HPUSH:		push	hl

NEXT:		ld	a, (bc)		;(W) <- ((IP))
		inc	bc		;(IP) <- (IP)+2
		ld	l, a
		ld	a, (bc)
		inc	bc
		ld	h, a		; (HL) <- CFA
NEXT1:		ld	e, (hl)		;(PC) <- ((W))
		inc	hl
		ld	d, (hl)
		ex	de, hl
		jp	(hl)		; NOTE: (DE) = CFA+1

;--------------------------------------------------
;		FORTH DICTIONARY
;
;
;	DICTIONARY FORMAT:
;
;				BYTE
;	ADDRESS	NAME		CONTENTS
;	------- ----		--------
;					  ( MSB=1
;					  ( P=PRECEDENCE BIT
;					  ( S=SMUDGE BIT
;	NFA	NAME FIELD	1PS<LEN>  < NAME LENGTH
;				0<1CHAR>  MSB=0, NAME'S 1ST CHAR
;				0<2CHAR>
;				  ...
;				1<LCHAR>  MSB=1, NAME'S LAST CHR
;	LFA	LINK FIELD	<LINKLB>  = PREVIOUS WORD'S NFA
;				<LINKHB>
;LABEL:	CFA	CODE FIELD	<CODELB>  = ADDR CPU CODE
;				<CODEHB>
;	PFA	PARAMETER	<1PARAM>  1ST PARAMETER BYTE
;		FIELD		<2PARAM>
;				  ...
;
;

DP0:		equ	$

;------------------------------------------------------------------------------
;LIT
		header	LIT,"LIT"

		ld	a, (bc)		; (HL) <- ((IP)) = LITERAL
		inc	bc		; (IP) <- (IP) + 2
		ld	l, a		; LB
		ld	a, (bc)		; HB
		inc	bc
		ld	h, a
		jp	HPUSH		; (S1) <- (HL)

;------------------------------------------------------------------------------
;EXECUTE
		header	EXEC,"EXECUTE"

		pop	hl		; (HL) <- (S1) = CFA
		jp	NEXT1

;------------------------------------------------------------------------------
;BRANCH
		header	BRAN,"BRANCH"

BRAN1:		ld	h, b		; (HL) <- (IP)
		ld	l, c
		ld	e, (hl)		; (DE) <- ((IP)) = BRANCH OFFSET
		inc	hl
		ld	d, (hl)
		dec	hl
		add	hl, de		; (HL) <- (HL) + ((IP))
		ld	c, l		; (IP) <- (HL)
		ld	b, h
		jp	NEXT

;------------------------------------------------------------------------------
;0BRANCH
		header	ZBRAN,"0BRANCH"

		pop	hl
		ld	a, l
		or	h
		jr	z, BRAN1	; IF (S1)=0 THEN BRANCH
		inc	bc		; ELSE SKIP BRANCH OFFSE
		inc	bc
		jp	NEXT

;------------------------------------------------------------------------------
; (LOOP)
		header	XLOOP,"(LOOP)"

		ld	de, 1		; (DE) <- INCREMENT
XLOO1:		ld	a, (iy+0)	; INDEX <- INDEX + INCR
		add	a, e
		ld	(iy+0),	a	
		ld	e, a
		ld	a, (iy+1)
		adc	a, d
		ld	(iy+1),	a	; LIMIT
		inc	d
		dec	d
		ld	d, a		; (DE) <- NEW INDEX
		jp	m, XLOO2	; IF INCR > 0
		ld	a, e
		sub	(iy+2)		; THEN (A) <- INDEX - LIMIT
		ld	a, d
		sbc	a, (iy+3)	
		jp	XLOO3
XLOO2:		ld	a, (iy+2)	; ELSE (A) <- LIMIT - INDEX
		sub	e
		ld	a, (iy+3)
		sbc	a, d
XLOO3:		jp	m, BRAN1	; IF (A) < 0 THEN LOOP AGAIN
		ld	de, 4
		add	iy, de
		inc	bc		; SKIP BRANCH OFFSET
		inc	bc	
		jp	NEXT

;------------------------------------------------------------------------------
; (+LOOP)
		header	XPLOO,"(+LOOP)"

		pop	de		; (DE) <- INCR
		jr	XLOO1

;------------------------------------------------------------------------------
; (DO)
		header	XDO,"(DO)"

		ld	de, 0FFFCh	; (RP) <- (RP) - 4
		add	iy, de
		pop	de		; (R1) <- (S1) = INIT INDEX
		ld	(iy+0),	e
		ld	(iy+1),	d
		pop	de		; (R2) <- (S2) = LIMIT
		ld	(iy+2),	e
		ld	(iy+3),	d
		jp	NEXT

;------------------------------------------------------------------------------
; I
		header	IDO,"I"

IDO1:		ld	l, (iy+0)	; HL <- (R1)
		ld	h, (iy+1)
		jp	HPUSH		; (S1) <- HL

;------------------------------------------------------------------------------
;DIGIT
		header	DIGIT,"DIGIT"

		pop	hl		; (L) <- (S1)LB = ASCII CHR TO BE CONVERTED
		pop	de		; (DE) <- (S2) = BASE VALUE
		ld	a, e
		sub	30h 		; IF CHR > "0"
		jp	m, DIGI2
		cp	0Ah		; AND IF CHR > "9"
		jp	m, DIGI1
		sub	7
		cp	0Ah		; AND IF CHR >= "A"
		jp	m, DIGI2
					; THEN VALID NUMERIC OR ALPHA CHR
DIGI1:		cp	l		; IF < BASE VALUE
		jp	p, DIGI2
					; THEN VALID DIGIT CHR
		ld	e, a		; (S2) <- (DE) = CONVERTED DIGIT
		ld	l, 1		; (S1) <- TRUE
		jp	DPUSH
					; ELSE INVALID DIGIT CHR
DIGI2:		ld	l, h		; (HL) <- FALSE
		jp	HPUSH		; (S1) <- FALSE

;------------------------------------------------------------------------------
; (FIND)
		header	PFIND,"(FIND)"
					; (2-1)FAILURE, (2-3)SUCCESS
		pop	de		; (DE) <- NFA
PFIN1:		pop	hl		; (HL) <- STRING ADDR
		push	hl		; SAVE STRING ADDR FOR NEXT ITERATIO
		ld	a, (de)
		xor	(hl)		; CHECK LENGTHS & SMUDGE BIT
		and	3Fh
		jr	nz, PFIN4	; LENGTHS DIFFERENT
					; LENGTHS MATCH, CHECK EACH CHR
PFIN2:		inc	hl		; (HL) <- ADDR NEXT CHR IN STRING
		inc	de		; (DE) <- ADDR NEXT CHR IN NF
		ld	a, (de)
		xor	(hl)		; IGNORE MSB
		add	a, a
		jr	nz, PFIN3	; NO MATCH
		jr	nc, PFIN2	; MATCH SO FAR, LOOP AGAIN
		ld	hl, 5		; STRING MATCHES
		add	hl, de		; ((SP)) <- PFA
		ex	(sp), hl
					; BACK UP TO LENGTH BYTE OF NF = NFA
PFIN6:		dec	de
		ld	a, (de)
		or	a
		jp	p, PFIN6	; IF MSB = 1 THEN (DE) = NFA
		ld	e, a		; (DE) <- LENGTH BYTE
		ld	d, 0
		ld	hl, 1		; (HL) <- TRUE
		jp	DPUSH		; RETURN, NF FOUND
; ABOVE NF NOT A MATCH, TRY ANOTHER
PFIN3:		jr	c, PFIN5	; IF NOT END OF NF
PFIN4:		inc	de		; THEN FIND END OF NF
		ld	a, (de)
		or	a
		jp	p, PFIN4
PFIN5:		inc	de		; (DE) <- LFA
		ex	de, hl
		ld	e, (hl)		; (DE) <- (LFA)
		inc	hl
		ld	d, (hl)
		ld	a, e
		or	d		; IF (LFA) <> 0
		jr	nz, PFIN1	; THEN TRY PREVIOUS DICT. DEF.
					; ELSE END OF DICTIONARY
		pop	hl		; DISCARD STRING ADDR
		ld	hl, 0		; (HL) <- FALSE
		jp	HPUSH		; RETURN, NO MATCH FOUND

;------------------------------------------------------------------------------
; ENCLOSE
		header	ENCL,"ENCLOSE"

		pop	de		; (DE) <- (S1) = DELIMITER CHAR
		pop	hl		; (HL) <- (S2) = ADDR TEXT TO SCAN
		push	hl		; (S4) <- ADDR
		ex	af, af'
		ld	a, e		; DELIM CHR
		ld	de, -1		; INITIALIZE CHR OFFSET COUNTER
		dec	hl		; (HL) <- ADDR-1
					; SKIP OVER LEADING DELIMITER CHRS
ENCL1:		inc	hl
		inc	de
		cp	(hl)		; IF TEXT CHR = DELIM CHR
		jr	z, ENCL1	; THEN LOOP AGAIN
					; ELSE NON-DELIM CHR FOUND
		push	de
		ex	af, af'	
		ld	a, (hl)		; IF 1ST NON-DELIM = NULL
		or	a
		jr	nz, ENCL2
					; THEN (S2) <- OFFSET TO BYTE
		inc	de		;   FOLLOWING NULL
		push	de
		dec	de		; (S1) <- OFFSET TO NULL
		push	de
		jp	NEXT
ENCL2:		ex	af, af'		; (A) <- DELIM CHR
		inc	hl		; (HL) <- ADDR NEXT CHR
		inc	de		; (E) <- OFFSET TO NEXT CHR
		cp	(hl)		; IF NEXT CHR <> DELIM CHR
		jr	z, ENCL4
		ex	af, af'
		ld	a, (hl)		; AND IF NEXT CHR <> NULL
		or	a
		jr	nz, ENCL2	; THEN CONTINUE SCAN
		push	de		; (S2) <- OFFSET TO NULL
		push	de		; (S1) <- OFFSET TO NULL
		jp	NEXT
					; ELSE CHR = DELIM CHR
ENCL4:		push	de		; (S2) <- OFFSET TO BYTE
					;   FOLLOWING TEXT
		inc	de		; (S1) <- OFFSET TO 2 BYTES AFTER
		; END OF WORD
		push	de
		jp	NEXT

;------------------------------------------------------------------------------
; EMIT
		header	EMIT,"EMIT",DOCOL

		dw	PEMIT
		dw	ONE
		dw	OUTT
		dw	PSTOR
		dw	SEMIS

;------------------------------------------------------------------------------
; (EMIT)	ORPHAN
PEMIT:		ld	h, b
		inc	b
		pop	hl
		ld	a, l
		call	COUT		; OUTPUT CHR TO CONSOLE
		jp	NEXT

;------------------------------------------------------------------------------
; KEY
		header	KEY,"KEY"

		call	CIN		; READ CHR FROM CONSOLE 
		ld	l, a
		ld	h, 0
		jp	HPUSH

;------------------------------------------------------------------------------
; ?TERMINAL
		header	QTERM,"?TERMINAL"

		call	CSTAT		; IF CHR TYPED
		ld	hl, 0
		or	a
		jr	z, PQTE1
		inc	hl		; IF CHR TYPED
PQTE1:		jp	HPUSH		; ELSE (S1) <- FALSE
;------------------------------------------------------------------------------
; CR
		header	CR,"CR"

		ld	a, 0Ah
		call	COUT
		ld	a, 0Dh
		call	COUT
		jp	NEXT
;------------------------------------------------------------------------------
; CMOVE
		header	CMOVE,"CMOVE"

		exx
		pop	bc
		pop	de
		pop	hl
		ld	a, b
		or	c
		jr	z, CMOV1
		ldir
CMOV1:		exx
		jp	NEXT
;------------------------------------------------------------------------------
; U* 16X16 UNSIGNED MULTIPLY
		header	USTAR,"U*"

		pop	hl
		exx
		pop	bc
		ld	de, 0
		ld	h, e
		ld	l, e
		exx
		ld	d, 10h
loc_4D2:	exx
		add	hl, hl
		rl	e
		rl	d
		exx
		add	hl, hl
		jr	nc, loc_4E2
		exx
		add	hl, bc
		jr	nc, loc_4E1
		inc	de
loc_4E1:	exx
loc_4E2:	dec	d
		jr	nz, loc_4D2
		exx
		push	hl
		push	de
		exx
		jp	NEXT
;------------------------------------------------------------------------------
; U/
		header	USLAS,"U/"

		ld	hl, 5
		add	hl, sp		; ((HL)) <- NUMERATOR.LW
		ld	d, (hl)		; (DE) <- NUMER.LW
		ld	(hl), b		; SAVE IP ON STACK
		dec	hl
		ld	e, (hl)
		ld	(hl), c
		pop	bc		; (BC) <- DENOMINATOR
		pop	hl		; (HL) <- NUMER.HW
		ld	a, l
		sub	c		; IF NUMER >= DENOM
		ld	a, h
		sbc	a, b
		jr	c, USLA1
		ld	hl, 0FFFFh	; THEN OVERFLOW
		ld	d, h		; SET REM & QUOT TO MAX
		ld	e, h
		jr	USLA7
USLA1:		ld	a, 16		; LOOP COUNTER
USLA2:		and	a		; LEFT SHIFT (HLDE) THRU CARRY
		rl	e
		rl	d
		rl	l
		rl	h
		jr	nc, USLA4
		and	a
		sbc	hl, bc
		jr	USLA5
USLA4:		sbc	hl, bc		; ELSE TRY (HL) <- (HL) - (BC)
		jr	nc, USLA5
		add	hl, bc		; UNDERFLOW, RESTORE
		dec	de
USLA5:		inc	de		; INC QUOT
		dec	a		; IF COUNTER > 0
		jr	nz, USLA2	; THEN LOOP AGAIN
USLA7:		pop	bc		; ELSE DONE, RESTORE IP
		push	hl		; (S2) <- REMAINDER
		push	de		; (S1) <- QUOTIENT
		jp	NEXT
;------------------------------------------------------------------------------
; AND
		header	ANDD,"AND"

		pop	hl		; (S1) <- (S1) AND (S2)
		pop	de
		ld	a, l
		and	e
		ld	l, a
		ld	a, h
		and	d
		ld	h, a
		jp	HPUSH
;------------------------------------------------------------------------------
; OR
		header	ORR,"OR"

		pop	hl		; (S1) <- (S1) OR (S2)
		pop	de
		ld	a, l
		or	e
		ld	l, a
		ld	a, h
		or	d
		ld	h, a
		jp	HPUSH
;------------------------------------------------------------------------------
; XOR
		header	XORR,"XOR"

		pop	hl		; (S1) <- (S1) XOR (S2)
		pop	de
		ld	a, l
		xor	e
		ld	l, a
		ld	a, h
		xor	d
		ld	h, a
		jp	HPUSH
;------------------------------------------------------------------------------
; SP@
		header	SPAT,"SP@"
					;(S1) <- (SP)
		ld	hl, 0
		add	hl, sp		; (HL) <- (SP)
		jp	HPUSH		; (S1) <- (HL)
;------------------------------------------------------------------------------
; SP!	STACK POINTER STORE
		header	SPSTO,"SP!"
					;(SP) <- (S0) ( USER VARIABLE )
		ld	hl, (UP)	; (HL) <- USER VAR BASE ADDR
		ld	de, 6
		add	hl, de		; (HL) <- S0
		ld	e, (hl)		; (DE) <- (S0)
		inc	hl
		ld	d, (hl)
		ex	de, hl
		ld	sp, hl		; (SP) <- (S0)
		jp	NEXT
;------------------------------------------------------------------------------
; RP@
		header	RPAT,"RP@"
					;(S1) <- (RP)
		push	iy		;RPP
		jp	NEXT
;------------------------------------------------------------------------------
; RP!	RETURN STACK POINTER STORE
		header	RPSTO,"RP!"
					;(RP) <- (R0) ( USER VARIABLE )
		ld	hl, (UP)	; (HL) <- USER VARIABLE BASE ADDR
		ld	de, 8
		add	hl, de		; (HL) <- R0
		ld	e, (hl)		; (DE) <- (R0)
		inc	hl
		ld	d, (hl)
		push	de
		pop	iy		; (RP) <- (R0)
		jp	NEXT
;------------------------------------------------------------------------------
; ;S
		header	SEMIS,";S"
					;(IP) <- (R1)
		ld	c, (iy+0)	; (BC) <- (R1)
		inc	iy
		ld	b, (iy+0)
		inc	iy		; (RP) <- (RP) + 2
		jp	NEXT
;------------------------------------------------------------------------------
; LEAVE
		header	LEAVE,"LEAVE"
					;LIMIT <- INDEX
		ld	a, (iy+0)	; (R2) LIMIT <- (R1) INDEX
		ld	(iy+2),	a
		ld	a, (iy+1)
		ld	(iy+3),	a
		jp	NEXT
;------------------------------------------------------------------------------
; >R
		header	TOR,">R"
					;(R1) <- (S1)
		dec	iy		; (RP) <- (RP) - 2
		dec	iy
		pop	de		; (DE) <- (S1)
		ld	(iy+0),	e	; (R1) <- (DE)
		ld	(iy+1),	d
		jp	NEXT
;------------------------------------------------------------------------------
; R>
		header	FROMR,"R>"
					;(S1) <- (R1)
		ld	l, (iy+0)	; (S1) <- (R1)
		inc	iy
		ld	h, (iy+0)
		inc	iy		; (RP) <- (RP) + 2
		jp	HPUSH
;------------------------------------------------------------------------------
; R
		header	RD,"R",IDO1			; wie I

;------------------------------------------------------------------------------
; 0=
		header	ZEQU,"0="

		pop	hl		; (HL) <- (S1)
		ld	a, h
		or	l		; IF (HL) = 0
		ld	hl, 0		; THEN (HL) <- FALSE
		jr	nz, ZEQU1
		inc	hl		; ELSE (HL) <- TRUE
ZEQU1:		jp	HPUSH		; (S1) <- (HL)
;------------------------------------------------------------------------------
; 0<
		header	ZLESS,"0<"

		pop	hl		; (S1) <- (HL)
		add	hl, hl		; IF (HL) >= 0
		ld	hl, 0		; THEN (HL) <- FALSE
		jr	nc, ZLES1
		inc	hl		; ELSE (HL) <- TRUE
ZLES1:		jp	HPUSH		; (S1) <- (HL)
;------------------------------------------------------------------------------
; +
		header	PLUS,"+"
					;(S1) <- (S1) + (S2)
		pop	de
		pop	hl
		add	hl, de
		jp	HPUSH
;------------------------------------------------------------------------------
; D+
		header	DPLUS,"D+"
					; D+	(4-2)
					; XLW XHW  YLW YHW  ---  SLW SHW
					; S4  S3   S2  S1        S2  S1
		ld	hl, 6
		add	hl, sp		; ((HL)) = XLW
		ld	e, (hl)		; (DE) = XLW
		ld	(hl), c		; SAVE IP ON STACK
		inc	hl
		ld	d, (hl)
		ld	(hl), b
		pop	bc		; (BC) <- YHW
		pop	hl		; (HL) <- YLW
		add	hl, de
		ex	de, hl		; (DE) <- YLW + XLW = SUM.LW
		pop	hl		; (HL) <- XHW
		ld	a, l
		adc	a, c
		ld	l, a		; (HL) <- YHW + XHW + CARRY
		ld	a, h
		adc	a, b
		ld	h, a
		pop	bc		; RESTORE IP
		push	de		; (S2) <- SUM.LW
		jp	HPUSH		; (S1) <- SUM.HW
;------------------------------------------------------------------------------
; MINUS
		header	MINUS,"MINUS"
					;(S1) <- -(S1)	( 2'S COMPLEMENT )
		ld	hl, 0
		pop	de
		or	a
		sbc	hl, de
		jp	HPUSH
;------------------------------------------------------------------------------
; DMINUS
		header	DMINU,"DMINUS"

		pop	hl		; (HL) <- HW
		pop	de		; (DE) <- LW
		xor	a
		sub	e		; (DE) <- 0 - (DE)
		ld	e, a
		ld	a, 0
		sbc	a, d
		ld	d, a
		ld	a, 0
		sbc	a, l		; (HL) <- 0 - (HL)
		ld	l, a
		ld	a, 0
		sbc	a, h
		ld	h, a
		jp	DPUSH		; (S2) <- LW, (S1) <- HW
;------------------------------------------------------------------------------
; OVER
		header	OVER,"OVER"

		pop	de
		pop	hl
		push	hl
		jp	DPUSH
;------------------------------------------------------------------------------
; DROP
		header	DROP,"DROP"

		pop	hl
		jp	NEXT
;------------------------------------------------------------------------------
; SWAP
		header	SWAP,"SWAP"

		pop	hl
		ex	(sp), hl
		jp	HPUSH
;------------------------------------------------------------------------------
; DUP
		header	DUPP,"DUP"

		pop	hl
		push	hl
		jp	HPUSH
;------------------------------------------------------------------------------
; 2DUP
		header	TDUP,"2DUP"

		pop	hl
		pop	de
		push	de
		push	hl
		jp	DPUSH
;------------------------------------------------------------------------------
; PLUS STORE
		header	PSTOR,"+!"
					;((S1)) <- ((S1)) + (S2)
		pop	hl		; (HL) <- (S1) = ADDR
		pop	de		; (HL) <- (S1) = ADDR
		ld	a, (hl)		; ((HL)) <- ((HL)) + (DE)
		add	a, e
		ld	(hl), a
		inc	hl
		ld	a, (hl)
		adc	a, d
		ld	(hl), a
		jp	NEXT

;------------------------------------------------------------------------------
;TOGGLE
		header	TOGGL,"TOGGLE"
					;((S2)) <- ((S2)) XOR (S1)LB
		pop	de		; (E) <- BYTE MASK
		pop	hl		; (E) <- BYTE MASK
		ld	a, (hl)
		xor	e
		ld	(hl), a		; (ADDR) <- (ADDR) XOR (E)
		jp	NEXT
;------------------------------------------------------------------------------
;AT
		header	AT,"@"
					;(S1) <- ((S1))
		pop	hl		; (HL) <- ADDR
		ld	e, (hl)		; (DE) <- (ADDR)
		inc	hl
		ld	d, (hl)
		push	de		; (S1) <- (DE)
		jp	NEXT
;------------------------------------------------------------------------------
;CAT
		header	CAT,"C@"
					;(S1) <- ((S1))LB
		pop	hl		; (HL) <- ADDR
		ld	l, (hl)		; (HL) <- (ADDR)LB
		ld	h, 0
		jp	HPUSH
;------------------------------------------------------------------------------
;2AT
		header	TAT,"2@"

		pop	hl		; (HL) <- ADDR HW
		ld	e, (hl)		; (DE) <- HW
		inc	hl
		ld	d, (hl)
		inc	hl
		ld	a, (hl)		; (HL) <- ADDR LW
		inc	hl
		ld	h, (hl)
		ld	l, a
		push	hl		; (S2) <- LW
		push	de		; (S1) <- HW
		jp	NEXT
;------------------------------------------------------------------------------
;STORE
		header	STORE,"!"
					;((S1)) <- (S2)
		pop	hl		; (HL) <- (S1) = ADDR
		pop	de		; (DE) <- (S2) = VALUE
		ld	(hl), e		; ((HL)) <- (DE)
		inc	hl
		ld	(hl), d
		jp	NEXT
;------------------------------------------------------------------------------
;CSTOR
		header	CSTOR,"C!"
					;((S1))LB <- (S2)LB
		pop	hl		; (HL) <- (S1) = ADDR
		pop	de		; (DE) <- (S2) = BYTE
		ld	(hl), e		; ((HL))LB <- (E)
		jp	NEXT
;------------------------------------------------------------------------------
;2!
		header	TSTOR,"2!"

		pop	hl		; (HL) <- ADDR
		pop	de		; (DE) <- HW
		ld	(hl), e		; (ADDR) <- HW
		inc	hl
		ld	(hl), d
		inc	hl		; (HL) <- ADDR LW
		pop	de		; (DE) <- LW
		ld	(hl), e		; (ADDR+2) <- LW
		inc	hl
		ld	(hl), d
		jp	NEXT
;------------------------------------------------------------------------------
; : CREATE NEW COLON-DEFINITION UNTIL ';'  
; : :   ?EXEC !CSP CURRENT @ CONTEXT ! CREATE ] ;CODE   IMMEDIATE

		header	COLON,":",DOCOL,IMMEDIATE

		dw	QEXEC
		dw	SCSP
		dw	CURR
		dw	AT
		dw	CONT
		dw	STORE
		dw	CREAT
		dw	RBRAC
		dw	PSCOD
DOCOL:		dec	iy		; RP
		ld	(iy+0),	b	; (R1) <- (IP)
		dec	iy		; (RP) <- (RP) - 2
		ld	(iy+0),	c
		inc	de		; (DE) <- CFA+2 = (W)
		ld	c, e		; (IP) <- (DE) = (W)
		ld	b, d
		jp	NEXT
;------------------------------------------------------------------------------
; ;
; : ;   ?CSP COMPILE ;S SMUDGE [ ;   IMMEDIATE 

		header	SEMI,";",DOCOL,IMMEDIATE

		dw	QCSP
		dw	COMP
		dw	SEMIS
		dw	SMUDG
		dw	LBRAC
		dw	SEMIS
;------------------------------------------------------------------------------
; CONSTANT
; : CONSTANT   CREATE SMUDGE , ;CODE

		header	CON,"CONSTANT",DOCOL

		dw	CREAT
		dw	SMUDG
		dw	COMMA
		dw	PSCOD
DOCON:		inc	de		; (DE) <- PFA
		ex	de, hl
		ld	e, (hl)		; (DE) <- (PFA)
		inc	hl
		ld	d, (hl)
		push	de		; (S1) <- (PFA)
		jp	NEXT
;------------------------------------------------------------------------------
; VARIABLE
; : VARIABLE    CONSTANT  ;CODE

		header	VAR,"VARIABLE",DOCOL

		dw	CON
		dw	PSCOD
DOVAR:		inc	de		; (DE) <- PFA
		push	de		; (S1) <- PFA
		jp	NEXT
;------------------------------------------------------------------------------
; USER
; : : USER   CONSTANT  ;CODE 

		header	USER,"USER",DOCOL

		dw	CON
		dw	PSCOD
DOUSE:		inc	de		; (DE) <- PFA
		ex	de, hl	
		ld	e, (hl)		; (DE) <- USER VARIABLE OFFSET
		inc	hl
		ld	d, (hl)
		ld	hl, (UP)	; (HL) <- USER VARIABLE BASE ADDR
		add	hl, de		; (HL) <- (HL) + (DE)
		jp	HPUSH		; (S1) <- BASE + OFFSET
;------------------------------------------------------------------------------
; NEXT
		header	NXT,"NEXT",DOCON

		dw	NEXT
;------------------------------------------------------------------------------
; ZERO
; 00  CONSTANT  0 

		header	ZERO,"0",DOCON

		dw	0
;------------------------------------------------------------------------------
; ONE
; 01  CONSTANT  1 

		header	ONE,"1",DOCON

		dw	1
;------------------------------------------------------------------------------
; TWO
; 02  CONSTANT  2

		header	TWO,"2",DOCON

		dw	2
;------------------------------------------------------------------------------
; THREE
; 03  CONSTANT  3 

		header	THREE,"3",DOCON

		dw	3
;------------------------------------------------------------------------------
; BL
; HEX 20  CONSTANT  BL

		header	BL,"BL",DOCON

		dw	' '
;------------------------------------------------------------------------------
; C/L (	CHARACTERS/LINE	)
; 20  CONSTANT  C/L 

		header	CSLL,"C/L",DOCON

		dw	20h
;------------------------------------------------------------------------------
; B/BUF	( BYTES/BUFFER )
; 200    CONSTANT   B/BUF

		header	BBUF,"B/BUF",DOCON

		dw	200h
;------------------------------------------------------------------------------
; B/SCR	( BUFFERS/SCREEN )
;  1     CONSTANT  B/SCR 

		header	BSCR,"B/SCR",DOCON

		dw	1
;------------------------------------------------------------------------------
; +ORIGIN
		header	PORIG,"+ORIGIN",DOCOL

		dw	LIT
		dw	ORIG
		dw	PLUS
		dw	SEMIS

;------------------------------------------------------------------------------
;
;	USER VARIABLES
;
; S0
		header	SZERO,"S0",DOUSE

		dw	6
;------------------------------------------------------------------------------
; R0
		header	RZERO,"R0",DOUSE

		dw	8
;------------------------------------------------------------------------------
; TIB
		header	TIB,"TIB",DOUSE

		dw	0Ah

;------------------------------------------------------------------------------
; WIDTH
		header	WIDTH,"WIDTH",DOUSE

		dw	0Ch
;------------------------------------------------------------------------------
; WARN
		header	WARN,"WARNING",DOUSE

		dw	0Eh
;------------------------------------------------------------------------------
; FENCE
		header	FENCE,"FENCE",DOUSE

		dw	10h
;------------------------------------------------------------------------------
; DP
		header	DP,"DP",DOUSE

		dw	12h
;------------------------------------------------------------------------------
; VOC-LINK
		header	VOCL,"VOC-LINK",DOUSE

		dw	14h
;------------------------------------------------------------------------------
; FIRST
		header	FIRST,"FIRST",DOUSE

		dw	16h
;------------------------------------------------------------------------------
; LIMIT
		header	LIMIT,"LIMIT",DOUSE

		dw	18h
;------------------------------------------------------------------------------
; INIT
		header	INIT,"INIT",DOCON

		dw	unk_286
		dw	1Ah
;------------------------------------------------------------------------------
; BLK
		header	BLK,"BLK",DOUSE

		dw	1Eh
;------------------------------------------------------------------------------
; IN
		header	INN,"IN",DOUSE

		dw	20h
;------------------------------------------------------------------------------
; OUT
		header	OUTT,"OUT",DOUSE

		dw	22h
;------------------------------------------------------------------------------
; SCR
		header	SCR,"SCR",DOUSE

		dw	24h
;------------------------------------------------------------------------------
; OFFSET
		header	OFSET,"OFFSET",DOUSE

		dw	1Ch
;------------------------------------------------------------------------------
; CONTEXT
		header	CONT,"CONTEXT",DOUSE

		dw	26h
;------------------------------------------------------------------------------
; CURRENT
		header	CURR,"CURRENT",DOUSE

		dw	28h
;------------------------------------------------------------------------------
; STATE
		header	STATE,"STATE",DOUSE

		dw	2Ah
;------------------------------------------------------------------------------
; BASE
		header	BASE,"BASE",DOUSE

		dw	2Ch
;------------------------------------------------------------------------------
; DPL
		header	DPL,"DPL",DOUSE

		dw	2Eh
;------------------------------------------------------------------------------
; FLD
		header	FLD,"FLD",DOUSE

		dw	30h
;------------------------------------------------------------------------------
; CSP
		header	CSPP,"CSP",DOUSE

		dw	32h
;------------------------------------------------------------------------------
; RNUM
		header	RNUM,"R#",DOUSE

		dw	34h
;------------------------------------------------------------------------------
; HLD
		header	HLD,"HLD",DOUSE

		dw	36h
;------------------------------------------------------------------------------
; REC
		header	REC,"REC",DOUSE

		dw	38h
;
;	END OF USER VARIABLES
;
;------------------------------------------------------------------------------
; 1+
		header	ONEP,"1+",DOCOL

		dw	ONE
		dw	PLUS
		dw	SEMIS
;------------------------------------------------------------------------------
; 2+
		header	TWOP,"2+",DOCOL

		dw	TWO
		dw	PLUS
		dw	SEMIS
;------------------------------------------------------------------------------
; HERE
		header	HERE,"HERE",DOCOL

		dw	DP
		dw	AT
		dw	SEMIS
;------------------------------------------------------------------------------
; ALLOT
		header	ALLOT,"ALLOT",DOCOL

		dw	DP
		dw	PSTOR
		dw	SEMIS
;------------------------------------------------------------------------------
; COMMA
		header	COMMA,",",DOCOL

		dw	HERE
		dw	TWO
		dw	ALLOT
		dw	STORE
		dw	SEMIS
;------------------------------------------------------------------------------
; CCOMM
; : C,   HERE 1 ALLOT C! ;

		header	CCOMM,"C,",DOCOL

		dw	HERE
		dw	ONE
		dw	ALLOT
		dw	CSTOR
		dw	SEMIS
;------------------------------------------------------------------------------
; SUBB
		header	SUBB,"-"

		pop	de		; (DE) <- (S1) = Y
		pop	hl		; (HL) <- (S2) = X
		and	a
		sbc	hl, de
		jp	HPUSH		; (S1) <- X - Y
;------------------------------------------------------------------------------
; EQUAL
; : =   - 0= ;
		header	EQUAL,"=",DOCOL

		dw	SUBB
		dw	ZEQU
		dw	SEMIS
;------------------------------------------------------------------------------
; LESS
		header	LESS,"<"
					; S2  <  S1
		pop	de		; (DE) <- (S1) = Y
		pop	hl		; (HL) <- (S2) = X
		ld	a, d		; IF X & Y HAVE SAME SIGNS
		xor	h
		jp	m, LES1
		and	a
		sbc	hl, de		; (HL) <- X - Y
LES1:		inc	h		; IF (HL) >= 0
		dec	h
		jp	m, LES2	
		ld	hl, 0		; THEN X >= Y
		jp	HPUSH		; (S1) <- FALSE
LES2:		ld	hl, 1		; ELSE X < Y
		jp	HPUSH		; (S1) <- TRUE
;------------------------------------------------------------------------------
; ULESS
; : U<   2DUP XOR 0< IF DROP 0< 0= ELSE - 0< ENDIF ;

		header	ULESS,"U<",DOCOL

		dw	TDUP
		dw	XORR
		dw	ZLESS
		dw	ZBRAN
		dw	ULES1-$		; IF
		dw	DROP
		dw	ZLESS
		dw	ZEQU
		dw	BRAN
		dw	ULES2-$
ULES1		dw	SUBB		; ELSE
		dw	ZLESS		
ULES2		dw	SEMIS		; ENDIF

;------------------------------------------------------------------------------
; >
		header	GREAT,">",DOCOL

		dw	SWAP
		dw	LESS
		dw	SEMIS
;------------------------------------------------------------------------------
; ROT
		header	ROT,"ROT"

		pop	de
		pop	hl
		ex	(sp), hl
		jp	DPUSH
;------------------------------------------------------------------------------
; SPACE
		header	SPACE,"SPACE",DOCOL

		dw	BL
		dw	EMIT
		dw	SEMIS
;------------------------------------------------------------------------------
; DDUP
		header	DDUP,"-DUP",DOCOL

		dw	DUPP
		dw	ZBRAN
		dw	4
		dw	DUPP
		dw	SEMIS
;------------------------------------------------------------------------------
; TRAVERSE
; : TRAVERSE   SWAP BEGIN OVER + 7F OVER C@ < UNTIL SWAP DROP ;

		header	TRAV,"TRAVERSE",DOCOL

		dw	SWAP
TRAV1		DW	OVER		; BEGIN
		dw	PLUS
		dw	LIT
		dw	7Fh
		dw	OVER
		dw	CAT
		dw	LESS
		dw	ZBRAN		; UNTIL
		dw	TRAV1-$
		dw	SWAP
		dw	DROP
		dw	SEMIS
;------------------------------------------------------------------------------
; LATEST
; : LATEST   CURRENT  @  @  ; 

		header	LATES,"LATEST",DOCOL

		dw	CURR
		dw	AT
		dw	AT
		dw	SEMIS
;------------------------------------------------------------------------------
; LFA
		header	LFA,"LFA",DOCOL

		dw	LIT
		dw	4
		dw	SUBB
		dw	SEMIS
;------------------------------------------------------------------------------
; CFA
		header	CFA,"CFA",DOCOL

		dw	TWO
		dw	SUBB
		dw	SEMIS

;------------------------------------------------------------------------------
;NFA
		header	NFA,"NFA",DOCOL

		dw	LIT
		dw	5
		dw	SUBB
		dw	LIT
		dw	0FFFFh
		dw	TRAV
		dw	SEMIS

;------------------------------------------------------------------------------
;PFA
		header	PFA,"PFA",DOCOL

		dw	ONE
		dw	TRAV
		dw	LIT
		dw	5
		dw	PLUS
		dw	SEMIS

;------------------------------------------------------------------------------
; STORE	CSP
		header	SCSP,"!CSP",DOCOL

		dw	SPAT
		dw	CSPP
		dw	STORE
		dw	SEMIS

;------------------------------------------------------------------------------
; ?ERROR
		header	QERR,"?ERROR",DOCOL

		dw	SWAP
		dw	ZBRAN		; IF
		dw	QERR1-$
		dw	ERROR
		dw	BRAN		; ELSE
		dw	QERR2-$
QERR1		dw	DROP		; ENDIF
QERR2		dw	SEMIS
;------------------------------------------------------------------------------
; ?COMP
		header	QCOMP,"?COMP",DOCOL

		dw	STATE
		dw	AT
		dw	ZEQU
		dw	LIT
		dw	11h
		dw	QERR
		dw	SEMIS
;------------------------------------------------------------------------------
; ?EXEC
		header	QEXEC,"?EXEC",DOCOL

		dw	STATE
		dw	AT
		dw	LIT
		dw	12h
		dw	QERR
		dw	SEMIS
;------------------------------------------------------------------------------
; ?PAIRS
		header	QPAIR,"?PAIRS",DOCOL

		dw	SUBB
		dw	LIT
		dw	13h
		dw	QERR
		dw	SEMIS
;------------------------------------------------------------------------------
;?CSP
		header	QCSP,"?CSP",DOCOL

		dw	SPAT
		dw	CSPP
		dw	AT
		dw	SUBB
		dw	LIT
		dw	14h
		dw	QERR
		dw	SEMIS
;------------------------------------------------------------------------------
;?LOADING
		header	QLOAD,"?LOADING",DOCOL

		dw	BLK
		dw	AT
		dw	ZEQU
		dw	LIT
		dw	16h
		dw	QERR
		dw	SEMIS
;------------------------------------------------------------------------------
;COMPILE
		header	COMP,"COMPILE",DOCOL

		dw	QCOMP
		dw	FROMR
		dw	DUPP
		dw	TWOP
		dw	TOR
		dw	AT
		dw	COMMA
		dw	SEMIS
;------------------------------------------------------------------------------
;LBRAC
		header	LBRAC,"((",DOCOL,IMMEDIATE

		dw	ZERO
		dw	STATE
		dw	STORE
		dw	SEMIS
;------------------------------------------------------------------------------
;RBRAC
		header	RBRAC,"))",DOCOL

		dw	LIT
		dw	0C0h
		dw	STATE
		dw	STORE
		dw	SEMIS
;------------------------------------------------------------------------------
;SMUDGE
		header	SMUDG,"SMUDGE",DOCOL

		dw	LATES
		dw	LIT
		dw	20h
		dw	TOGGL
		dw	SEMIS
;------------------------------------------------------------------------------
;HEX
		header	HEX,"HEX",DOCOL

		dw	LIT
		dw	10h
		dw	BASE
		dw	STORE
		dw	SEMIS
;------------------------------------------------------------------------------
;DEC
		header	DEC,"DECIMAL",DOCOL

		dw	LIT
		dw	0Ah
		dw	BASE
		dw	STORE
		dw	SEMIS
;------------------------------------------------------------------------------
; (;CODE)
		header	PSCOD,"(;CODE)",DOCOL

		dw	FROMR
		dw	LATES
		dw	PFA
		dw	CFA
		dw	STORE
		dw	SEMIS
;------------------------------------------------------------------------------
;SEMIC
		header	SEMIC,";CODE",DOCOL,IMMEDIATE

		dw	QCSP
		dw	COMP
		dw	PSCOD
		dw	LBRAC
SEMI1		dw	TASK		; 
		dw	SEMIS
;------------------------------------------------------------------------------
; <BUILDS
		header	BUILD,"<BUILDS",DOCOL

		dw	ZERO
		dw	CON
		dw	SEMIS
;------------------------------------------------------------------------------
;DOES
		header	DOES,"DOES>",DOCOL

		dw	FROMR
		dw	LATES
		dw	PFA
		dw	STORE
		dw	PSCOD
DODOE:		dec	iy		; RP
		ld	(iy+0),	b	; (R1) <- (IP) = PFA = (SUBSTITUTE CFA)
		dec	iy		; (RP) <- (RP) - 2
		ld	(iy+0),	c
		inc	de		; (DE) <- PFA = (SUBSTITUTE CFA)
		ex	de, hl
		ld	c, (hl)		; (IP) <- (SUBSTITUTE CFA)
		inc	hl
		ld	b, (hl)
		inc	hl
		jp	HPUSH		; (S1) <- PFA+2 = SUBSTITUTE PFA
;------------------------------------------------------------------------------
;COUNT
; : COUNT   DUP 1+ SWAP C@  ; 

		header	COUNT,"COUNT",DOCOL

		dw	DUPP
		dw	ONEP
		dw	SWAP
		dw	CAT
		dw	SEMIS
;------------------------------------------------------------------------------
;TYPE
;  TYPE   -DUP  IF OVER + SWAP DO I C@ EMIT LOOP  ELSE DROP ENDIF ;

		header	TYPE,"TYPE",DOCOL

		dw	DDUP
		dw	ZBRAN		; IF
		dw	TYPE1-$
		dw	OVER
		dw	PLUS
		dw	SWAP
		dw	XDO		; DO
TYPE2		dw	IDO
		dw	CAT
		dw	EMIT
		dw	XLOOP		; LOOP
		dw	TYPE2-$
		dw	BRAN		; ELSE
		DW	TYPE3-$
TYPE1		DW	DROP		; ENDIF
TYPE3		DW	SEMIS
;------------------------------------------------------------------------------
; -TRAILING
; -TRAILING   DUP 0 DO OVER OVER + 1 - C@ BL - IF LEAVE ELSE 1 - ENDIF LOOP ;
       
		header	DTRAI,"-TRAILING",DOCOL

		dw	DUPP
		dw	ZERO
		dw	XDO		; DO
DTRA1		dw	OVER
		dw	OVER
		dw	PLUS
		dw	ONE
		dw	SUBB
		dw	CAT
		dw	BL
		dw	SUBB
		dw	ZBRAN		; IF
		dw	DTRA2-$
		dw	LEAVE
		dw	BRAN		; ELSE
		dw	DTRA3-$
DTRA2		dw	ONE
		dw	SUBB		; ENDIF
DTRA3		dw	XLOOP		; LOOP
		dw	DTRA1-$
		dw	SEMIS
;------------------------------------------------------------------------------
; (.")
; : (.")   R COUNT DUP 1+ R> + >R TYPE ;
       
		header	PDOTQ,"(.\i)",DOCOL

		dw	RD
		dw	COUNT
		dw	DUPP
		dw	ONEP
		dw	FROMR
		dw	PLUS
		dw	TOR
		dw	TYPE
		dw	SEMIS
;------------------------------------------------------------------------------
; ."
; : ."   22 STATE @ IF COMPILE (.") WORD HERE C@ 1+ ALLOT 
;                   ELSE WORD HERE COUNT TYPE ENDIF ; IMMEDIATE

		header	DOTQ,".\i",DOCOL,IMMEDIATE

		dw	LIT
		dw	22h
		dw	STATE
		dw	AT
		dw	ZBRAN		; IF
		dw	DOTQ1-$
		dw	COMP
		dw	PDOTQ
		dw	WORD
		dw	HERE
		dw	CAT
		dw	ONEP
		dw	ALLOT
		dw	BRAN		; ELSE
		dw	DOTQ2-$
DOTQ1		dw	WORD
		dw	HERE
		dw	COUNT
		dw	TYPE		; ENDIF
DOTQ2		dw	SEMIS
;------------------------------------------------------------------------------
; EXPECT
		header	EXPEC,"EXPECT"

EXPEC1:		exx
		in	a, (PIO_A)
		set	2, a		; IRM einschalten
		out	(PIO_A), a
EXPEC2:		call	PV1
		DB	KBD		; Tasteneingabe mit Cursoreinblendung
		cp	0Dh		; Enter-Taste?
		jr	z, EXPEC3
		call	PV1
		DB	CRT		; Zeichen auf Bildschirm anzeigen
		jr	EXPEC2
EXPEC3:		ld	de, (CURSO)
		ld	a, d
		or	e
		jr	z, EXPEC4
		ld	a, 8		; Cursor nach links
		call	PV1
		DB	CRT		; Zeichen auf Bildschirm anzeigen
		ld	de, (CURSO)
		call	PV1
		DB	DABR		; Video-RAM-Adresse berechnen
		ld	a,(hl)		; Zeichencode aus VRAM lesen
		or	a
		jr	nz, EXPEC3
		ld	a, 9		; Cursor nach rechts
		call	PV1
		DB	CRT		; Zeichen auf Bildschirm anzeigen
EXPEC4:		pop	bc
EXPEC5:		ld	de, (CURSO)
		call	PV1
		DB	DABR		; Video-RAM-Adresse berechnen
		ld	d,(hl)
		ld	a,b
		or	c
		jr	z, EXPEC6
		ld	a, d
		or	a
		jr	z, EXPEC6
		pop	hl
		ld	(hl), d
		inc	hl
		push	hl
		ld	a, 9		; Cursor nach rechts
		call	PV1
		DB	CRT		; Zeichen auf Bildschirm anzeigen
		dec	bc
		jr	EXPEC5
EXPEC6:		ld	a, 9		; Cursor nach rechts
		call	PV1
		DB	CRT		; Zeichen auf Bildschirm anzeigen
		pop	hl
		ld	(hl), 0
		in	a, (PIO_A)
		RES	2, a		; IRM ausschalten
		out	(PIO_A), a
		exx
		jp	NEXT
;------------------------------------------------------------------------------
; QUERY
		header	QUERY,"QUERY",DOCOL

		dw	TIB
		dw	AT
		dw	LIT
		dw	50h
		dw	EXPEC
		dw	ZERO
		dw	INN
		dw	STORE
		dw	SEMIS
;------------------------------------------------------------------------------
; THE NULL WORD
		header	NULL,"\000",DOCOL,IMMEDIATE

		dw	BLK
		dw	AT
		dw	ZBRAN		; IF
		dw	NULL1-$
		dw	ONE
		dw	BLK
		dw	PSTOR
		dw	ZERO
		dw	INN
		dw	STORE
		dw	BLK
		dw	AT
		dw	BSCR
		dw	ONE
		dw	SUBB
		dw	ANDD
		dw	ZEQU
		dw	ZBRAN		; IF
		dw	NULL2-$
		dw	QEXEC
		dw	FROMR
		dw	DROP		; ENDIF
NULL2		DW	BRAN		; ELSE
		DW	NULL3-$
NULL1		DW	FROMR
		DW	DROP		; ENDIF
NULL3		DW	SEMIS
;------------------------------------------------------------------------------
; FILL
		header	FILL,"FILL"

		ld	l, c
		ld	h, b
		pop	de
		pop	bc
		ex	(sp), hl
		ex	de, hl
FILL1:		ld	a, b		; BEGIN
		or	c
		jr	z, FILL2	; WHILE
		ld	a, l
		ld	(de), a
		inc	de
		dec	bc
		jr	FILL1		; REPEAT
FILL2:		pop	bc
		jp	NEXT
;------------------------------------------------------------------------------
; ERASE
		header	ERASE,"ERASE",DOCOL

		dw	ZERO
		dw	FILL
		dw	SEMIS
;------------------------------------------------------------------------------
; BLANK
		header	BLANK,"BLANKS",DOCOL

		dw	BL
		dw	FILL
		dw	SEMIS
;------------------------------------------------------------------------------
; HOLD
		header	HOLD,"HOLD",DOCOL

		dw	LIT
		dw	0FFFFh
		dw	HLD
		dw	PSTOR
		dw	HLD
		dw	AT
		dw	CSTOR
		dw	SEMIS
;------------------------------------------------------------------------------
; PAD
		header	PAD,"PAD",DOCOL

		dw	HERE
		dw	LIT
		dw	44h
		dw	PLUS
		dw	SEMIS
;------------------------------------------------------------------------------
; WORD
		header	WORD,"WORD",DOCOL

		dw	BLK
		dw	AT
		dw	ZBRAN		; IF
		dw	WORD1-$
		dw	BLK
		dw	AT
		dw	BLOCK
		dw	BRAN		; ELSE
		dw	WORD2-$
WORD1		dw	TIB
		dw	AT		; ENDIF
WORD2		dw	INN
		dw	AT
		dw	PLUS
		dw	SWAP
		dw	ENCL
		dw	HERE
		dw	LIT
		dw	22h
		dw	BLANK
		dw	INN
		dw	PSTOR
		dw	OVER
		dw	SUBB
		dw	TOR
		dw	RD
		dw	HERE
		dw	CSTOR
		dw	PLUS
		dw	HERE
		dw	ONEP
		dw	FROMR
		dw	CMOVE
		dw	SEMIS
;------------------------------------------------------------------------------
; (NUMBER)
		header	PNUMB,"(NUMBER)",DOCOL

PNUM1		dw	ONEP		; BEGIN
		dw	DUPP
		dw	TOR
		dw	CAT
		dw	BASE
		dw	AT
		dw	DIGIT
		dw	ZBRAN		; WHILE
		dw	PNUM2-$
		dw	SWAP
		dw	BASE
		dw	AT
		dw	USTAR
		dw	DROP
		dw	ROT
		dw	BASE
		dw	AT
		dw	USTAR
		dw	DPLUS
		dw	DPL
		dw	AT
		dw	ONEP
		dw	ZBRAN		; IF
		dw	PNUM3-$
		dw	ONE
		dw	DPL
		dw	PSTOR		; ENDIF
PNUM3		DW	FROMR
		DW	BRAN		; REPEAT
		DW	PNUM1-$
PNUM2		DW	FROMR
		DW	SEMIS
;------------------------------------------------------------------------------
; NUMBER
		header	NUMB,"NUMBER",DOCOL

		dw	ZERO
		dw	ZERO
		dw	ROT
		dw	DUPP
		dw	ONEP
		dw	CAT
		dw	LIT
		dw	'-'
		dw	EQUAL
		dw	DUPP
		dw	TOR
		dw	PLUS
		dw	LIT
		dw	-1
NUMB1		dw	DPL		; BEGIN
		dw	STORE
		dw	PNUMB
		dw	DUPP
		dw	CAT
		dw	BL
		dw	SUBB
		dw	ZBRAN		; WHILE
		dw	NUMB2-$
		dw	DUPP
		dw	CAT
		dw	LIT
		dw	'.'
		dw	SUBB
		dw	ZERO
		dw	QERR
		dw	ZERO
		dw	BRAN		; REPEAT
		dw	NUMB1-$
NUMB2		dw	DROP
		dw	FROMR
		dw	ZBRAN		; IF
		DW	NUMB3-$
		DW	DMINU		; ENDIF
NUMB3		DW	SEMIS
;------------------------------------------------------------------------------
; -FIND
		header	DFIND,"-FIND",DOCOL
					; -FIND	(0-3) SUCCESS
					; (0-1) FAILURE
		dw	BL
		dw	WORD
		dw	HERE
		dw	CONT
		dw	AT
		dw	AT
		dw	PFIND
		dw	DUPP
		dw	ZEQU
		dw	ZBRAN		; IF
		DW	DFIN1-$
		dw	DROP
		dw	HERE
		dw	LATES
		dw	PFIND		; ENDIF
DFIN1		DW	SEMIS
;------------------------------------------------------------------------------
; (ABORT)
		header	PABOR,"(ABORT)",DOCOL

		dw	ABORT
		dw	SEMIS
;------------------------------------------------------------------------------
; ERROR
		header	ERROR,"ERROR",DOCOL

		dw	WARN
		dw	AT
		dw	ZLESS
		dw	ZBRAN		; IF
		dw	ERRO1-$
		dw	PABOR		; ENDIF
ERRO1		dw	HERE
		dw	COUNT
		dw	TYPE
		dw	PDOTQ
		db	2,"? "
		dw	MESS
		dw	SPSTO
		dw	BLK
		dw	AT
		dw	DDUP
		dw	ZBRAN
		dw	ERRO2-$		; IF
		dw	INN
		dw	AT
		dw	SWAP		; ENDIF
ERRO2		dw	QUIT
;------------------------------------------------------------------------------
; ID.
		header	IDDOT,"ID.",DOCOL

		dw	PAD
		dw	LIT
		dw	20h
		dw	LIT
		dw	5Fh
		dw	FILL
		dw	DUPP
		dw	PFA
		dw	LFA
		dw	OVER
		dw	SUBB
		dw	PAD
		dw	SWAP
		dw	CMOVE
		dw	PAD
		dw	COUNT
		dw	LIT
		dw	1Fh
		dw	ANDD
		dw	TYPE
		dw	SPACE
		dw	SEMIS
;------------------------------------------------------------------------------
; CREATE
		header	CREAT,"CREATE",DOCOL

		dw	DFIND
		dw	ZBRAN		; IF
		dw	CREA1-$
		dw	DROP
		dw	NFA
		dw	IDDOT
		dw	LIT
		dw	4
		dw	MESS
		dw	SPACE		; ENDIF
CREA1		dw	HERE
		dw	DUPP
		dw	CAT
		dw	WIDTH
		dw	AT
		dw	MIN
		dw	ONEP
		dw	ALLOT
		dw	DUPP
		dw	LIT
		dw	0A0h
		dw	TOGGL
		dw	HERE
		dw	ONE
		dw	SUBB
		dw	LIT
		dw	80h
		dw	TOGGL
		dw	LATES
		dw	COMMA
		dw	CURR
		dw	AT
		dw	STORE
		dw	HERE
		dw	TWOP
		dw	COMMA
		dw	SEMIS
;------------------------------------------------------------------------------
; (COMPILE)
		header	BCOMP,"(COMPILE)",DOCOL,IMMEDIATE

		dw	DFIND
		dw	ZEQU
		dw	ZERO
		dw	QERR
		dw	DROP
		dw	CFA
		dw	COMMA
		dw	SEMIS
;------------------------------------------------------------------------------
; LITERAL
		header	LITER,"LITERAL",DOCOL,IMMEDIATE

		dw	STATE
		dw	AT
		dw	ZBRAN		; IF
		dw	LITE1-$
		dw	COMP
		dw	LIT
		dw	COMMA		; ENDIF
LITE1		dw	SEMIS
;------------------------------------------------------------------------------
; DLITERAL
		header	DLITE,"DLITERAL",DOCOL,IMMEDIATE

		dw	STATE
		dw	AT
		dw	ZBRAN		; IF
		dw	DLIT1-$
		dw	SWAP
		dw	LITER
		dw	LITER		; ENDIF
DLIT1		dw	SEMIS
;------------------------------------------------------------------------------
; ?STACK
		header	QSTAC,"?STACK",DOCOL

		dw	SPAT
		dw	SZERO
		dw	AT
		dw	SWAP
		dw	ULESS
		dw	ONE
		dw	QERR
		dw	SPAT
		dw	ZERO
		dw	LIT
		dw	20h
		dw	PLUS
		dw	ULESS
		dw	LIT
		dw	7
		dw	QERR
		dw	SEMIS
;------------------------------------------------------------------------------
; INTERPRET
; : INTERPRET   BEGIN -FIND IF STATE @ < IF CFA , ELSE CFA EXECUTE ENDIF ?STACK 
;              ELSE HERE NUMBER DPL @ 1+ IF [COMPILE] DLITERAL ELSE DROP 
;              [COMPILE] LITERAL ENDIF ?STACK ENDIF AGAIN ; 
       
		header	INTER,"INTERPRET",DOCOL

INTE1		DW	DFIND		; BEGIN
		DW	ZBRAN		; IF
		DW	INTE2-$
		DW	STATE
		DW	AT
		DW	LESS
		DW	ZBRAN		; IF
		DW	INTE3-$
		DW	CFA
		DW	COMMA
		DW	BRAN		; ELSE
		DW	INTE4-$
INTE3		DW	CFA
		DW	EXEC		; ENDIF
INTE4		DW	QSTAC
		DW	BRAN		; ELSE
		DW	INTE5-$
INTE2		DW	HERE
		DW	NUMB
		DW	DPL
		DW	AT
		DW	ONEP
		DW	ZBRAN		; IF
		DW	INTE6-$
		DW	DLITE
		DW	BRAN		; ELSE
		DW	INTE7-$
INTE6		DW	DROP
		DW	LITER		; ENDIF
INTE7		DW	QSTAC		; ENDIF
INTE5		DW	BRAN		; AGAIN
		DW	INTE1-$
;------------------------------------------------------------------------------
; IMMEDIATE
		header	IMMED,"IMMEDIATE",DOCOL

		dw	LATES
		dw	LIT
		dw	40h
		dw	TOGGL
		dw	SEMIS
;------------------------------------------------------------------------------
; VOCABULARY
; : VOCABULARY   <BUILDS A081 , CURRENT @ CFA , HERE VOC-LINK @ , VOC-LINK ! 
;                DOES> 2+ CONTEXT ! ;
       
		header	VOCAB,"VOCABULARY",DOCOL

		dw	BUILD
		dw	LIT
		dw	0A081h
		dw	COMMA
		dw	CURR
		dw	AT
		dw	CFA
		dw	COMMA
		dw	HERE
		dw	VOCL
		dw	AT
		dw	COMMA
		dw	VOCL
		dw	STORE
		dw	DOES
DOVOC:		dw	TWOP
		dw	CONT
		dw	STORE
		dw	SEMIS
;------------------------------------------------------------------------------
; FORTH
		header	FORTH,"FORTH",DODOE,IMMEDIATE

		dw	DOVOC
word_1171:	dw	0A081h
FORTHP:		dw	TASK_		; COLD START VALUE ONLY
					; CHANGED EACH TIME A DEF IS APPENDED
					; TO THE FORTH VOCABULARY
word_1175:	dw	0		; END OF VOCABULARY LIST
;------------------------------------------------------------------------------
; DEFINITIONS
		header	DEFIN,"DEFINITIONS",DOCOL

		dw	CONT
		dw	AT
		dw	CURR
		dw	STORE
		dw	SEMIS
;------------------------------------------------------------------------------
; (
unk_1191	header	PAREN,"(",DOCOL,IMMEDIATE

		dw	LIT
		dw	29h
		dw	WORD
		dw	SEMIS
;------------------------------------------------------------------------------
; QUIT
		header	QUIT,"QUIT",DOCOL

		dw	ZERO
		dw	BLK
		dw	STORE
		dw	LBRAC
QUIT1		DW	RPSTO		; BEGIN
		DW	CR
		DW	QUERY
		DW	INTER
		DW	STATE
		DW	AT
		DW	ZEQU
		DW	ZBRAN		; IF
		DW	QUIT2-$
		DW	PDOTQ
		DB	2,"OK"		; ENDIF
QUIT2		DW	BRAN		; AGAIN
		DW	QUIT1-$
;------------------------------------------------------------------------------
; ABORT
		header	ABORT,"ABORT",DOCOL

		dw	SPSTO
		dw	DEC
		dw	QSTAC
		dw	CR
		dw	PDOTQ
		db	10,"HC - FORTH"
		dw	FORTH
		dw	DEFIN
		dw	QUIT
;------------------------------------------------------------------------------

WRM:		ld	bc, WRM1
		jp	NEXT

WRM1:		dw	WARM
;------------------------------------------------------------------------------
; WARM
		header	WARM,"WARM",DOCOL

		dw	PABOR

;------------------------------------------------------------------------------
CLD:		ld	bc, CLD1
		ld	hl, (ORIG+12H)		; SP
		ld	sp, hl
		ld	iy, (ORIG+14H)		; RP
		ld	hl, (ORIG+10H)		; UP
		ld	(UP), hl
		jp	NEXT
;------------------------------------------------------------------------------
CLD1:		dw	COLD
; COLD
		header	COLD,"COLD",DOCOL

		dw	LIT
		dw	ORIG+12H
		dw	LIT
		dw	UP
		dw	AT
		dw	LIT
		dw	6
		dw	PLUS
		dw	LIT
		dw	18h
		dw	CMOVE
		dw	LIT
		dw	ORIG+0CH
		dw	AT
		dw	LIT
		dw	FORTHP
		dw	STORE
		dw	INIT
		dw	AT
		dw	ZLESS
		dw	ZBRAN		; IF
		dw	16h
		dw	FORTH
		dw	DEFIN
		dw	DEC
		dw	ONE
		dw	NOBUF
		dw	BSCR
		dw	SLASH
		dw	CLOAD
		dw	THREE
		dw	LOAD
		dw	PABOR
;------------------------------------------------------------------------------
; S->D
		header	STOD,"S->D"

		pop	de
		ld	hl, 0
		ld	a, d
		and	80h
		jr	z, STOD1
		dec	hl
STOD1:		jp	DPUSH
;------------------------------------------------------------------------------
; +-
		header	PM,"+-",DOCOL

		dw	ZLESS
		DW	ZBRAN		; IF
		DW	PM1-$
		DW	MINUS		; ENDIF
PM1		DW	SEMIS
;------------------------------------------------------------------------------
; D+-
		header	DPM,"D+-",DOCOL

		dw	ZLESS
		DW	ZBRAN		; IF
		DW	DPM1-$
		DW	DMINU		; ENDIF
DPM1		DW	SEMIS
;------------------------------------------------------------------------------
;ABS
		header	ABS,"ABS",DOCOL

		dw	DUPP
		dw	PM
		dw	SEMIS
;------------------------------------------------------------------------------
;DABS
		header	DABS,"DABS",DOCOL

		dw	DUPP
		dw	DPM
		dw	SEMIS
;------------------------------------------------------------------------------
;MIN
		header	MIN,"MIN",DOCOL

		dw	TDUP
		dw	GREAT
		dw	ZBRAN
		DW	MIN1-$
		DW	SWAP		; ENDIF
MIN1		DW	DROP
		DW	SEMIS
;------------------------------------------------------------------------------
;MAX
		header	MAX,"MAX",DOCOL

		dw	TDUP
		dw	LESS
		dw	ZBRAN
		DW	MAX1-$
		DW	SWAP		; ENDIF
MAX1		DW	DROP
		DW	SEMIS
;------------------------------------------------------------------------------
; M*
		header	MSTAR,"M*",DOCOL

		dw	TDUP
		dw	XORR
		dw	TOR
		dw	ABS
		dw	SWAP
		dw	ABS
		dw	USTAR
		dw	FROMR
		dw	DPM
		dw	SEMIS
;------------------------------------------------------------------------------
; M/
		header	MSLAS,"M/",DOCOL

		dw	OVER
		dw	TOR
		dw	TOR
		dw	DABS
		dw	RD
		dw	ABS
		dw	USLAS
		dw	FROMR
		dw	RD
		dw	XORR
		dw	PM
		dw	SWAP
		dw	FROMR
		dw	PM
		dw	SWAP
		dw	SEMIS
;------------------------------------------------------------------------------
; *
		header	STAR,"*",DOCOL

		dw	MSTAR
		dw	DROP
		dw	SEMIS
;------------------------------------------------------------------------------
; /MOD
		header	SLMOD,"/MOD",DOCOL

		dw	TOR
		dw	STOD
		dw	FROMR
		dw	MSLAS
		dw	SEMIS
;------------------------------------------------------------------------------
; /
		header	SLASH,"/",DOCOL

		dw	SLMOD
		dw	SWAP
		dw	DROP
		dw	SEMIS
;------------------------------------------------------------------------------
; MOD
		header	MODD,"MOD",DOCOL

		dw	SLMOD
		dw	DROP
		dw	SEMIS
;------------------------------------------------------------------------------
; */MOD
		header	SSMOD,"*/MOD",DOCOL

		dw	TOR
		dw	MSTAR
		dw	FROMR
		dw	MSLAS
		dw	SEMIS
;------------------------------------------------------------------------------
; */
		header	SSLA,"*/",DOCOL

		dw	SSMOD
		dw	SWAP
		dw	DROP
		dw	SEMIS
;------------------------------------------------------------------------------
;M/MOD
		header	MSMOD,"M/MOD",DOCOL

		dw	TOR
		dw	ZERO
		dw	RD
		dw	USLAS
		dw	FROMR
		dw	SWAP
		dw	TOR
		dw	USLAS
		dw	FROMR
		dw	SEMIS
;------------------------------------------------------------------------------
; (LINE)
		header	PLINE,"(LINE)",DOCOL

		dw	LIT
		dw	10h
		dw	STAR
		dw	PLUS
		dw	LIT
		dw	10h
		dw	SLMOD
		dw	TOR
		dw	CSLL
		dw	BBUF
		dw	SSMOD
		dw	FROMR
		dw	BSCR
		dw	STAR
		dw	PLUS
		dw	BLOCK
		dw	PLUS
		dw	CSLL
		dw	SEMIS
;------------------------------------------------------------------------------
; .LINE
		header	DLINE,".LINE",DOCOL

		dw	PLINE
		dw	DTRAI
		dw	TYPE
		dw	SEMIS
;------------------------------------------------------------------------------
;MESSAGE
		header	MESS,"MESSAGE",DOCOL

		dw	WARN
		dw	AT
		dw	ZBRAN		; IF
		dw	MESS1-$
		dw	DDUP
		dw	ZBRAN		; IF
		dw	MESS2-$
		dw	ONE
		dw	DLINE
		DW	SPACE		; ENDIF
MESS2		DW	BRAN		; ELSE
		DW	MESS3-$
MESS1		dw	PDOTQ
		db	6,"MSG # "
		DW	DOT		; ENDIF
MESS3		DW	SEMIS
;------------------------------------------------------------------------------
;
;	FORTH VARIABLES AND CONSTANTS USED IN BLOCK INTERFACE
;
;REC/BLK
		header	RCBLK,"REC/BLK",DOCON

		dw	4
;------------------------------------------------------------------------------
; #BUFF	 ( NUMBER OF BUFFERS )
		header	NOBUF,"#BUFF",DOCOL

		dw	LIMIT
		dw	AT
		dw	FIRST
		dw	AT
		dw	SUBB
		dw	BBUF
		dw	TWOP
		dw	SLASH
		dw	SEMIS
;------------------------------------------------------------------------------
;
;	BLOCK INTERFACE HIGH-LEVEL ROUTINES
;
; EMPTY-BUFFERS
		header	MTBUF,"EMPTY-BUFFERS",DOCOL

		dw	FIRST
		dw	AT
		dw	LIMIT
		dw	AT
		dw	OVER
		dw	SUBB
		dw	ERASE
		dw	SEMIS
;------------------------------------------------------------------------------
; BLOCK
		header	BLOCK,"BLOCK",DOCOL

		dw	ONE
		dw	MAX
		dw	NOBUF
		dw	MIN
		dw	ONE
		dw	SUBB
		dw	BBUF
		dw	TWOP
		dw	STAR
		dw	FIRST
		dw	AT
		dw	PLUS
		dw	SEMIS
;------------------------------------------------------------------------------
; LOAD
		header	LOAD,"LOAD",DOCOL

		dw	BLK
		dw	AT
		dw	TOR
		dw	INN
		dw	AT
		dw	TOR
		dw	ZERO
		dw	INN
		dw	STORE
		dw	BSCR
		dw	STAR
		dw	BLK
		dw	STORE		; BLK <- SCR * B/SCR
		dw	INTER		; INTERPRET FROM OTHER SCREEN
		dw	FROMR
		dw	INN
		dw	STORE
		dw	FROMR
		dw	BLK
		dw	STORE
		dw	SEMIS
;------------------------------------------------------------------------------
; -->
		header	ARROW,"-->",DOCOL,IMMEDIATE

		dw	QLOAD
		dw	ZERO
		dw	INN
		dw	STORE
		dw	BSCR
		dw	BLK
		dw	AT
		dw	OVER
		dw	MODD
		dw	SUBB
		dw	BLK
		dw	PSTOR
		dw	SEMIS
;------------------------------------------------------------------------------
; '     ( TICK )
		header	TICK,"'",DOCOL,IMMEDIATE

		dw	DFIND
		dw	ZEQU
		dw	ZERO
		dw	QERR
		dw	DROP
		dw	LITER
		dw	SEMIS
;------------------------------------------------------------------------------
; FORGET
		header	FORG,"FORGET",DOCOL

		dw	TICK
		dw	NFA
		dw	DUPP
		dw	FENCE
		dw	AT
		dw	ULESS
		dw	LIT
		dw	15h
		dw	QERR
		dw	TOR
		dw	VOCL
		dw	AT
		dw	RD
		dw	OVER
		dw	ULESS
		dw	ZBRAN
		dw	12h
		dw	FORTH
		dw	DEFIN
		dw	AT
		dw	DUPP
		dw	VOCL
		dw	STORE
		dw	BRAN
		dw	0FFE8h
		dw	DUPP
		dw	LIT
		dw	4
		dw	SUBB
		dw	PFA
		dw	LFA
		dw	AT
		dw	DUPP
		dw	RD
		dw	ULESS
		dw	ZBRAN
		dw	0FFF2h
		dw	OVER
		dw	TWO
		dw	SUBB
		dw	STORE
		dw	AT
		dw	DDUP
		dw	ZEQU
		dw	ZBRAN
		dw	0FFD8h
		dw	FROMR
		dw	DP
		dw	STORE
		dw	SEMIS
;------------------------------------------------------------------------------
; BACK
		header	BACK,"BACK",DOCOL

		dw	HERE
		dw	SUBB
		dw	COMMA
		dw	SEMIS
;------------------------------------------------------------------------------
; BEGIN
		header	BEGIN,"BEGIN",DOCOL,IMMEDIATE

		dw	QCOMP
		dw	HERE
		dw	ONE
		dw	SEMIS
;------------------------------------------------------------------------------
; ENDIF
		header	ENDIFF,"ENDIF",DOCOL,IMMEDIATE

		dw	QCOMP
		dw	TWO
		dw	QPAIR
		dw	HERE
		dw	OVER
		dw	SUBB
		dw	SWAP
		dw	STORE
		dw	SEMIS
;------------------------------------------------------------------------------
; DO
		header	DO,"DO",DOCOL,IMMEDIATE

		dw	COMP
		dw	XDO
		dw	HERE
		dw	THREE
		dw	SEMIS
;------------------------------------------------------------------------------
; LOOP
		header	LOOP,"LOOP",DOCOL,IMMEDIATE

		dw	THREE
		dw	QPAIR
		dw	COMP
		dw	XLOOP
		dw	BACK
		dw	SEMIS
;------------------------------------------------------------------------------
; +LOOP
		header	PLOOP,"+LOOP",DOCOL,IMMEDIATE

		dw	THREE
		dw	QPAIR
		dw	COMP
		dw	XPLOO
		dw	BACK
		dw	SEMIS
;------------------------------------------------------------------------------
; UNTIL
		header	UNTIL,"UNTIL",DOCOL,IMMEDIATE

		dw	ONE
		dw	QPAIR
		dw	COMP
		dw	ZBRAN
		dw	BACK
		dw	SEMIS
;------------------------------------------------------------------------------
; AGAIN
		header	AGAIN,"AGAIN",DOCOL,IMMEDIATE

		dw	ONE
		dw	QPAIR
		dw	COMP
		dw	BRAN
		dw	BACK
		dw	SEMIS
;------------------------------------------------------------------------------
; REPEAT
		header	REPEA,"REPEAT",DOCOL,IMMEDIATE

		dw	TOR
		dw	TOR
		dw	AGAIN
		dw	FROMR
		dw	FROMR
		dw	TWO
		dw	SUBB
		dw	ENDIFF
		dw	SEMIS
;------------------------------------------------------------------------------
; IF
		header	IFF,"IF",DOCOL,IMMEDIATE

		dw	COMP
		dw	ZBRAN
		dw	HERE
		dw	ZERO
		dw	COMMA
		dw	TWO
		dw	SEMIS
;------------------------------------------------------------------------------
; ELSE
		header	ELSEE,"ELSE",DOCOL,IMMEDIATE

		dw	TWO
		dw	QPAIR
		dw	COMP
		dw	BRAN
		dw	HERE
		dw	ZERO
		dw	COMMA
		dw	SWAP
		dw	TWO
		dw	ENDIFF
		dw	TWO
		dw	SEMIS
;------------------------------------------------------------------------------
; WHILE
		header	WHILE,"WHILE",DOCOL,IMMEDIATE

		dw	IFF
		dw	TWOP
		dw	SEMIS
;------------------------------------------------------------------------------
;SPACES
		header	SPACS,"SPACES",DOCOL

		dw	ZERO
		dw	MAX
		dw	DDUP
		DW	ZBRAN		; IF
		DW	SPAX1-$
		DW	ZERO
		DW	XDO		; DO
SPAX2		DW	SPACE
		DW	XLOOP		; LOOP	ENDIF
		DW	SPAX2-$
SPAX1		DW	SEMIS
;------------------------------------------------------------------------------
; <#
		header	BDIGS,"<#",DOCOL

		dw	PAD
		dw	HLD
		dw	STORE
		dw	SEMIS
;------------------------------------------------------------------------------
; #>
		header	EDIGS,"#>",DOCOL

		dw	DROP
		dw	DROP
		dw	HLD
		dw	AT
		dw	PAD
		dw	OVER
		dw	SUBB
		dw	SEMIS
;------------------------------------------------------------------------------
; SIGN
		header	SIGN,"SIGN",DOCOL

		dw	ROT
		dw	ZLESS
		DW	ZBRAN		; IF
		DW	SIGN1-$
		DW	LIT
		DW	'-'
		DW	HOLD		; ENDIF
SIGN1		DW	SEMIS
;------------------------------------------------------------------------------
; #
		header	DIG,"#",DOCOL

		dw	BASE
		dw	AT
		dw	MSMOD
		dw	ROT
		dw	LIT
		dw	9
		dw	OVER
		dw	LESS
		dw	ZBRAN		; IF
		DW	DIG1-$
		DW	LIT
		DW	7
		DW	PLUS		; ENDIF
DIG1		DW	LIT
		dw	30h
		dw	PLUS
		dw	HOLD
		dw	SEMIS
;------------------------------------------------------------------------------
; #S
		header	DIGS,"#S",DOCOL

		dw	DIG
		dw	OVER
		dw	OVER
		dw	ORR
		dw	ZEQU
		dw	ZBRAN
		dw	0FFF4h
		dw	SEMIS
;------------------------------------------------------------------------------
; D.R
		header	DDOTR,"D.R",DOCOL

		dw	TOR
		dw	SWAP
		dw	OVER
		dw	DABS
		dw	BDIGS
		dw	DIGS
		dw	SIGN
		dw	EDIGS
		dw	FROMR
		dw	OVER
		dw	SUBB
		dw	SPACS
		dw	TYPE
		dw	SEMIS
;------------------------------------------------------------------------------
; .R
		header	DOTR,".R",DOCOL

		dw	TOR
		dw	STOD
		dw	FROMR
		dw	DDOTR
		dw	SEMIS
;------------------------------------------------------------------------------
; D.
		header	DDOT,"D.",DOCOL

		dw	ZERO
		dw	DDOTR
		dw	SPACE
		dw	SEMIS
;------------------------------------------------------------------------------
; .
		header	DOT,".",DOCOL

		dw	STOD
		dw	DDOT
		dw	SEMIS
;------------------------------------------------------------------------------
; ?
		header	QUES,"?",DOCOL

		dw	AT
		dw	DOT
		dw	SEMIS
;------------------------------------------------------------------------------
; U.
		header	UDOT,"U.",DOCOL

		dw	ZERO
		dw	DDOT
		dw	SEMIS
;------------------------------------------------------------------------------
; VLIST
		header	VLIST,"VLIST",DOCOL

		dw	BBUF
		dw	OUTT
		dw	STORE
		dw	CONT
		dw	AT
		dw	AT
		dw	DUPP
		dw	CAT
		dw	LIT
		dw	1Fh
		dw	ANDD
		dw	OUTT
		dw	AT
		dw	PLUS
		dw	LIT
		dw	24h
		dw	GREAT
		dw	ZBRAN
		dw	0Eh
		dw	CR
		dw	ZERO
		dw	OUTT
		dw	STORE
		dw	BRAN
		dw	6
		dw	SPACE
		dw	SPACE
		dw	DUPP
		dw	IDDOT
		dw	PFA
		dw	LFA
		dw	AT
		dw	DUPP
		dw	ZEQU
		dw	QTERM
		dw	ORR
		dw	ZBRAN
		dw	0FFC2h
		dw	DROP
		dw	SEMIS
;------------------------------------------------------------------------------
; LIST
		header	LIST,"LIST",DOCOL

		dw	DEC
		dw	CR
		dw	DUPP
		dw	SCR
		dw	STORE
		dw	PDOTQ
		db	6,"SCR # "
		dw	DOT
		dw	BBUF
		dw	BSCR
		dw	STAR
		dw	CSLL
		dw	SLASH
		dw	ZERO
		dw	XDO
LIST1		dw	CR
		dw	IDO
		dw	LIT
		dw	3
		dw	DOTR
		dw	SPACE
		dw	IDO
		dw	SCR
		dw	AT
		dw	DLINE
		dw	QTERM		; ?TERMINAL
		dw	ZBRAN		; IF
		dw	LIST2-$
		dw	LEAVE		; LEAVE
LIST2		dw	XLOOP
		dw	LIST1-$		; ENDIF
		dw	CR
		dw	SEMIS
;------------------------------------------------------------------------------
; INDEX
		header	INDEX,"INDEX",DOCOL

		dw	LIT
		dw	0Ch
		dw	EMIT
		dw	CR
		dw	ONEP
		dw	SWAP
		dw	XDO
		dw	CR
		dw	IDO
		dw	LIT
		dw	3
		dw	DOTR
		dw	SPACE
		dw	ZERO
		dw	IDO
		dw	DLINE
		dw	QTERM
		dw	ZBRAN
		dw	4
		dw	LEAVE
		dw	XLOOP
		dw	0FFE4h
		dw	SEMIS
;------------------------------------------------------------------------------
; TRIAD
		header	TRIAD,"TRIAD",DOCOL

		dw	LIT
		dw	0Ch
		dw	EMIT
		dw	LIT
		dw	3
		dw	SLASH
		dw	LIT
		dw	3
		dw	STAR
		dw	LIT
		dw	3
		dw	OVER
		dw	PLUS
		dw	SWAP
		dw	XDO
TRIA1		dw	CR
		dw	IDO
		dw	LIST
		dw	QTERM		; ?TERMINAL
		dw	ZBRAN		; IF
		dw	TRIA2-$
		dw	LEAVE
TRIA2		dw	XLOOP		; ENDIF
		dw	TRIA1-$
		dw	CR
		dw	LIT
		dw	15
		dw	MESS
		dw	CR
		dw	SEMIS
;------------------------------------------------------------------------------
; KC85 spezifisches
;------------------------------------------------------------------------------

; BUSIN
		header	BUSIN,"BUSIN"
		
		exx
		pop	hl
		ld	c, l
		in	a, (c)
		exx
		ld	l, a
		ld	h, 0
		jp	HPUSH
;------------------------------------------------------------------------------
; BUSOUT
		header	BUSOUT,"BUSOUT"
	
		exx
		pop	hl
		pop	de
		ld	c, l
		out	(c), e
		exx
		jp	NEXT
;------------------------------------------------------------------------------
; CAOS

		header	CAOS,"CAOS"
		
		pop	hl
		pop	de
		push	bc
		call	PV4		; Programmverteiler IV (UP-Nr. in Reg. E)
		pop	bc
		jp	NEXT
;------------------------------------------------------------------------------
;IC@
		header	ICAT,"IC@"

		pop	hl
		ld	e,LDAM		; 29h
		call	PV4		; Byte (A) aus Speicher (HL) lesen
		ld	l, a
		ld	h, 0
		jp	HPUSH
;------------------------------------------------------------------------------
;I@
		header	IAT,"I@"

		pop	hl
		ld	e,LDAM		; 29h
		call	PV4		; Byte (A) aus Speicher (HL) lesen
		ld	d, a		; gelesenes Byte
		inc	hl		; Adresse+1
		call	PV4		; Byte (A) aus Speicher (HL) lesen
		ld	e, d
		ld	d, a		; DE = gelesenes Wort
		push	de
		jp	NEXT
;------------------------------------------------------------------------------
;IC!
		header	ICSTOR,"IC!"

		pop	hl
		pop	de
		ld	a, e
		ld	e,LDMA		; 28h
		call	PV4		; Byte (A) in Speicher (HL) schreiben
		jp	NEXT
;------------------------------------------------------------------------------
;I!
		header	ISTOR,"I!"

		pop	hl
		pop	de
		ld	a, e		; Byte (Low)
		ld	e,LDMA		; 28h
		call	PV4		; Byte (A) in Speicher (HL) schreiben
		ld	a, d		; Byte (High)
		inc	hl		; Adresse+1
		call	PV4		; Byte (A) in Speicher (HL) schreiben
		jp	NEXT
;------------------------------------------------------------------------------
;BYE
		header	BYE,"BYE",DOCOL

		dw	CR
		dw	LIT
		dw	12h
		dw	ZERO
		dw	CAOS
		dw	SEMIS
;------------------------------------------------------------------------------
;LOC
		header	LOC,"LOC",DOCOL

		dw	LIT
		dw	CURSO
		dw	ICSTOR
		dw	LIT
		dw	CURSO+1
		dw	ICSTOR
		dw	SEMIS
;------------------------------------------------------------------------------
;WINDOW
		header	WINDOW,"WINDOW",DOCOL

		dw	ZERO
		dw	LIT
		dw	CURSO
		dw	ISTOR
		dw	OVER
		dw	OVER
		dw	PLUS
		dw	LIT
		dw	20h
		dw	GREAT
		dw	LIT
		dw	5
		dw	QERR
		dw	LIT
		dw	WINLG+1		; 0B79Fh
		dw	ICSTOR
		dw	LIT
		dw	WINON+1		; 0B79Dh
		dw	ICSTOR
		dw	OVER
		dw	OVER
		dw	PLUS
		dw	LIT
		dw	28h
		dw	GREAT
		dw	LIT
		dw	5
		dw	QERR
		dw	LIT
		dw	WINLG		; 0B79Eh
		dw	ICSTOR
		dw	LIT
		dw	WINON		; 0B79Ch
		dw	ICSTOR
		dw	SEMIS
;------------------------------------------------------------------------------

IRM_ON:		in	a, (PIO_A)
		set	2, a		; IRM einschalten
		out	(PIO_A), a
		ret

IRM_OFF:	in	a, (PIO_A)
		RES	2, a		; IRM ausschalten
		out	(PIO_A), a
		ret
;------------------------------------------------------------------------------
;PIX
		header	PIX,"PIX"

		pop	de
		pop	hl
		call	IRM_ON
		ld	a, e
		ld	(VERT), a	; Wert Y-Position
		ld	a, h
		and	1
		ld	h, a
		jr	z, PIX1
		ld	a, l
		and	3Fh
		ld	l, a
PIX1:		ld	(HOR), hl	; Wert X-Position
		call	IRM_OFF
		ld	e,PUDE		; 2Fh
		pop	hl
		ld	a, l
		or	a
		jr	z, PIX2
		inc	e		; 30h = PUSE
PIX2:		call	PV4		; Punkt setzen / loeschen
		jp	NEXT
;
;------------------------------------------------------------------------------
;INK
		header	INK,"INK"

		ld	hl, COLR	; Textfarbe
INK1:		pop	de
		call	IRM_ON
		ld	a, e
		add	a, a		; Vordergrundfarbe
		add	a, a		; nach Bit 3-6 rotieren
		add	a, a
		and	7Fh		; Blinken aus
		ld	d, a		; neue Vordergrundfarbe
		ld	a, (hl)		; alter Farbwert
		and	87h		; Vordergrundfarbe entfernen
		or	d		; Vordergrundfarbe einbauen
		ld	(hl), a		; Farbwert zurueckschreiben
		call	IRM_OFF
		jp	NEXT
;------------------------------------------------------------------------------
;INKP
		header	INKP,"INKP"

		ld	hl, FARB	; Grafikfarbe
		jr	INK1
;------------------------------------------------------------------------------
;BLINK
		header	BLINK,"BLINK"

		ld	hl, COLR	; Textfarbe
BLINK1:		call	IRM_ON
		ld	a, 80h		; Bit 7 = Blinken
		xor	(hl)		; Blinken ein/aus
		ld	(hl), a
		call	IRM_OFF
		jp	NEXT
;------------------------------------------------------------------------------
;BLINKP
		header	BLINKP,"BLINKP"

		ld	hl, FARB	; Grafikfarbe
		jr	BLINK1	; Blinken ein/aus
;------------------------------------------------------------------------------
;PAPER
		header	PAPER,"PAPER"
	
		ld	hl, COLR	; Textfarbe
		call	IRM_ON
		pop	de
		ld	a, e
		and	7		; Maske fuer Hintergrund: Bit 0-2
		ld	d, a
		ld	a, (hl)		; alter Farbwert
		and	0F8h		; Hintergrundfarbe entfernen
		or	d		; neue Hintergrundfarbe einbauen
		ld	(hl), a		; Farbwert zurueckschreiben
		call	IRM_OFF
		jp	NEXT
;------------------------------------------------------------------------------
;COLOR
		header	COLOR,"COLOR",DOCOL

		dw	PAPER
		dw	INK
		dw	SEMIS

;------------------------------------------------------------------------------
;SOUND
		header	SOUND,"SOUND"

		exx
		ld	hl, ARG3+1
		ld	b, 6		; 6 Bytes laden
		call	IRM_ON
SOUND1:		pop	de
		ld	(hl), e
		dec	hl
		djnz	SOUND1
		ld	e,TON		; 35h
		call	PV4		; Tonausgabe
		exx
		jp	NEXT
;------------------------------------------------------------------------------
;R/W
		header	RW,"R/W"

		pop	hl
		ld	a, l
		cp	5
		jr	c, RW1
		in	a, (PIO_A)
		and	9Fh		; LED und Kassettenrecorder ausschalten
		out	(PIO_A), a
		push	ix
		pop	hl
		ld	a, h
		ld	e,SIXD		; 31h
		call	PV4		; IX-Arbeitsbereich verlagern
		jp	NEXT
;
RW1:		call	IRM_ON
		ld	a, l
		exx
		cp	3
		jr	nc, RW2
		ld	e, 2
		jr	RW3
RW2:		ld	e, 0Ah		; Routine ISRO - Init Kassettenausgabe
RW3:		ld	d, 0
		ld	hl, (SUTAB)	; Zeiger auf Unterprogrammtabelle
		add	hl, de
		ld	e, (hl)		; Adresse von ISRO ermitteln
		inc	hl
		ld	d, (hl)
		ex	de, hl
		push	af
		call	IRM_OFF
		pop	af
		pop	de
		ld	(ix+5),	e	; Kassettenpuffer
		ld	(ix+6),	d	; Adresse eintragen
		cp	3
		jr	nc, RW8
		or	a
		jr	nz, RW4
		ld	(ix+2),	0	; Blocknummer 0
		ld	bc, 2000h	; langer Vorton
		jr	RW6
RW4:		cp	2
		jr	nz, RW5
		ld	(ix+2),	0FEh	; letzter Block
RW5:		ld	bc, 0E0h 	; kurzer Vorton
RW6:		in	a, (PIO_A)
		or	60h		; LED und Kassettenrecorder einschalten
		out	(PIO_A), a
		ld	de, RW7
		push	de
		jp	(hl)
		;
RW7:		exx
		jp	NEXT
RW8:		cp	3
		jr	nz, RW9
		in	a, (PIO_A)
		or	40h		; Kassettenrecorder ein
		and	0DFh		; LED aus
		out	(PIO_A), a
RW9:		ld	de, RW10
		push	de
		jp	(hl)
RW10:		exx
		ld	d, 0
		ld	e, (ix+2)
		ld	hl, 0
		adc	hl, hl
		jp	DPUSH
;------------------------------------------------------------------------------
; CLOAD
		header	CLOAD,"CLOAD",DOCOL

		dw	LIT
		dw	0Ch
		dw	EMIT
		dw	CR
		dw	PAD
		dw	THREE
		dw	RW
		dw	DROP
		dw	ONE
		dw	EQUAL
		dw	ZEQU
		dw	ZBRAN
		dw	0Ah
		dw	PDOTQ
		db	1,'*'
		dw	BRAN
		dw	0FFE8h
		dw	CR
		dw	PAD
		dw	LIT
		dw	0Bh
		dw	TYPE
		dw	CR
		dw	OFSET
		dw	AT
		dw	RCBLK
		dw	STAR
		dw	ONEP
		dw	REC
		dw	STORE
		dw	SWAP
		dw	ONE
		dw	MAX
		dw	DUPP
		dw	BLOCK
		dw	ROT
		dw	NOBUF
		dw	MIN
		dw	RCBLK
		dw	STAR
		dw	ROT
		dw	ONE
		dw	SUBB
		dw	RCBLK
		dw	STAR
		dw	XDO
		dw	ONE
		dw	REC
		dw	PSTOR
		dw	ZERO
		dw	THREE
		dw	ZERO
		dw	XDO
		dw	DROP
		dw	DUPP
		dw	LIT
		dw	4
		dw	RW
		dw	SWAP
		dw	DUPP
		dw	THREE
		dw	DOTR
		dw	DUPP
		dw	LIT
		dw	0FFh
		dw	EQUAL
		dw	IDO
		dw	ZEQU
		dw	ANDD
		dw	ZBRAN
		dw	8
		dw	DUPP
		dw	REC
		dw	STORE
		dw	REC
		dw	AT
		dw	EQUAL
		dw	ZEQU
		dw	ZBRAN
		dw	0Ch
		dw	PDOTQ
		db	1,'*'
		dw	DROP
		dw	BRAN
		dw	0FFC4h
		dw	DUPP
		dw	ZBRAN
		dw	0Ah
		dw	PDOTQ
		db	1,'?'
		dw	BRAN
		dw	8
		dw	LEAVE
		dw	PDOTQ
		db	1,'>'
		dw	XLOOP
		dw	0FFAAh
		dw	ZBRAN
		dw	10h
		dw	CR
		dw	LIT
		dw	8
		dw	MESS
		dw	REC
		dw	QUES
		dw	CR
		dw	LIT
		dw	80h
		dw	PLUS
		dw	REC
		dw	AT
		dw	RCBLK
		dw	MODD
		dw	ONE
		dw	EQUAL
		dw	ZBRAN
		dw	4
		dw	TWOP
		dw	REC
		dw	AT
		dw	LIT
		dw	0FFh
		dw	EQUAL
		dw	ZBRAN
		dw	4
		dw	LEAVE
		dw	XLOOP
		dw	0FF5Eh
		dw	DROP
		dw	LIT
		dw	5
		dw	RW
		dw	REC
		dw	AT
		dw	LIT
		dw	0FFh
		dw	SUBB
		dw	ZBRAN
		dw	0Ah
		dw	CR
		dw	LIT
		dw	6
		dw	MESS
		dw	SEMIS
;------------------------------------------------------------------------------
; VERIFY
		header	VERIF,"VERIFY",DOCOL

		dw	LIT
		dw	11h
		dw	ZERO
		dw	CAOS
		dw	SEMIS
;------------------------------------------------------------------------------
; CSAVE
		header	CSAVE,"CSAVE",DOCOL

		dw	BL
		dw	WORD
		dw	LIT
		dw	2029h		; ') '
		dw	LIT
		dw	4628h		; '(F'
		dw	HERE
		dw	LIT
		dw	9
		dw	PLUS
		dw	TSTOR
		dw	HERE
		dw	LIT
		dw	0Ch
		dw	PLUS
		dw	LIT
		dw	80h
		dw	ERASE
		dw	HERE
		dw	ONEP
		dw	ZERO
		dw	RW
		dw	NOBUF
		dw	MIN
		dw	DUPP
		dw	ROT
		dw	ONE
		dw	MAX
		dw	OVER
		dw	OVER
		dw	SUBB
		dw	ZBRAN
		dw	28h
		dw	XDO
		dw	IDO
		dw	BLOCK
		dw	RCBLK
		dw	ZERO
		dw	XDO
		dw	DUPP
		dw	ONE
		dw	RW
		dw	LIT
		dw	80h
		dw	PLUS
		dw	XLOOP
		dw	0FFF2h
		dw	DROP
		dw	XLOOP
		dw	0FFE2h
		dw	BRAN
		dw	6
		dw	DROP
		dw	DROP
		dw	BLOCK
		dw	THREE
		dw	ZERO
		dw	XDO
		dw	DUPP
		dw	ONE
		dw	RW
		dw	LIT
		dw	80h
		dw	PLUS
		dw	XLOOP
		dw	0FFF2h
		dw	TWO
		dw	RW
		dw	LIT
		dw	5
		dw	RW
		dw	SEMIS
;------------------------------------------------------------------------------
;PLOT
		header	PLOT,"PLOT"

		call	IRM_ON
		exx
		ld	a, 30h
		pop	de
		pop	bc
		exx
		pop	de
		pop	hl
		exx
		pop	hl
		dec	iy
		ld	(iy+0),	l
		bit	7, d
		jr	nz, PLOT1
		or	0Ah
		jr	PLOT2
PLOT1:		and	a
		ld	hl, 0
		sbc	hl, de
		ex	de, hl
PLOT2:		bit	7, b
		jr	nz, PLOT3
		or	5
		jr	PLOT4
PLOT3:		and	a
		ld	hl, 0
		sbc	hl, bc
		ld	b, h
		ld	c, l
PLOT4:		ld	h, b
		ld	l, c
		and	a
		sbc	hl, de
		jr	c, PLOT5
		push	bc
		and	0DFh
		jr	PLOT6
PLOT5:		ld	h, d
		ld	l, e
		and	a
		sbc	hl, bc
		push	de
		and	0EFh
		ld	e, c
		ld	d, b
PLOT6:		ld	c, l
		ld	b, h
		srl	h
		rr	l
PLOT7:		exx
		ex	af, af'
		push	hl
		ld	a, h
		and	1
		ld	h, a
		jr	z, PLOT8
		ld	a, l
		and	3Fh ; '?'
		ld	l, a
PLOT8:		ld	(HOR), hl	; X-Position
		pop	hl
		ld	a, e
		ld	(VERT), a	; Y-Position
		ld	a, (iy+0)
		push	de
		ld	e, 30h		; PUSE = Punkt setzen
		dec	a
		jr	z, PLOT9
		dec	e		; 2FH = PUDE, Punkt loeschen
PLOT9:		call	PV3
		pop	de
		ex	af, af'
		exx
		ex	(sp), hl
		dec	hl
		bit	7, h
		jr	nz, PLOT18
		ex	(sp), hl
		and	a
		sbc	hl, de
		jr	c, PLOT13
		exx
		bit	4, a
		jr	z, PLOT11
		bit	0, a
		jr	z, PLOT10
		inc	hl
		jr	PLOT11
PLOT10:		dec	hl
PLOT11:		bit	5, a
		jr	z, PLOT17
		bit	1, a
		jr	z, PLOT12
		inc	de
		jr	PLOT17
PLOT12:		dec	de
		jr	PLOT17
PLOT13:		add	hl, de
		add	hl, bc
		exx
		bit	2, a
		jr	z, PLOT14
		inc	hl
		jr	PLOT15
PLOT14:		dec	hl
PLOT15:		bit	3, a
		jr	z, PLOT16
		inc	de
		jr	PLOT17
PLOT16:		dec	de
PLOT17:		exx
		jr	PLOT7
PLOT18:		ld	h, 0
		ld	l, (iy+0)
		ex	(sp), hl
		inc	iy
		exx
		push	hl
		push	de
		call	IRM_OFF
		jp	NEXT
;------------------------------------------------------------------------------
; Vocabulary EDITOR
; Zeilenorientierter FIG-Text-Editor
;------------------------------------------------------------------------------
; EDITOR
unk_1DC2
		header	EDITOR,"EDITOR",DODOE,IMMEDIATE

		dw	DOVOC
		dw	0A081h
		dw	unk_1F2C
off_1DD3:	dw	word_1175
;------------------------------------------------------------------------------
; TEXT
unk_1DD5
		header	TEXT,"TEXT",DOCOL,,word_1171	
					; lfa auf word_1171, Vokabular EDITOR

		dw	HERE
		dw	CSLL
		dw	ONEP
		dw	BLANK
		dw	WORD
		dw	HERE
		dw	PAD
		dw	CSLL
		dw	ONEP
		dw	CMOVE
		dw	SEMIS
;------------------------------------------------------------------------------
; LINE
		header	LINE,"LINE",DOCOL

		dw	DUPP
		dw	LIT
		dw	0FFF0h
		dw	ANDD
		dw	LIT
		dw	17h
		dw	QERR
		dw	SCR
		dw	AT
		dw	PLINE
		dw	DROP
		dw	SEMIS
;------------------------------------------------------------------------------
; MOVE
		header	MOVE,"-MOVE",DOCOL

		dw	LINE
		dw	CSLL
		dw	CMOVE
		dw	SEMIS
;------------------------------------------------------------------------------
; R
		header	off_1E2B,"R",DOCOL

		dw	PAD
		dw	ONEP
		dw	SWAP
		dw	MOVE
		dw	SEMIS
;------------------------------------------------------------------------------
; P
		header	off_1E3B,"P",DOCOL

		dw	ONE
		dw	TEXT
		dw	off_1E2B
		dw	SEMIS
;------------------------------------------------------------------------------
; H
		header	off_1E49,"H",DOCOL

		dw	LINE
		dw	PAD
		dw	ONEP
		dw	CSLL
		dw	CMOVE
		dw	SEMIS
;------------------------------------------------------------------------------
; T
		header	off_1E5B,"T",DOCOL

		dw	SCR
		dw	AT
		dw	DLINE
		dw	SEMIS
;------------------------------------------------------------------------------
; E
		header	off_1E69,"E",DOCOL

		dw	LINE
		dw	CSLL
		dw	BLANK
		dw	SEMIS
;------------------------------------------------------------------------------
; L
		header	off_1E77,"L",DOCOL

		dw	LIT
		dw	0Ch
		dw	EMIT
		dw	SCR
		dw	AT
		dw	LIST
		dw	SEMIS
;------------------------------------------------------------------------------
; S
		header	off_1E8B,"S",DOCOL

		dw	LIT
		dw	0Eh
		dw	LIT
		dw	0Fh
		dw	ROT
		dw	XDO
		dw	DUPP
		dw	LINE
		dw	OVER
		dw	ONEP
		dw	MOVE
		dw	ONE
		dw	SUBB
		dw	XLOOP
		dw	0FFF0h
		dw	ONEP
		dw	off_1E69
		dw	SEMIS
;------------------------------------------------------------------------------
; D
		header	off_1EB5,"D",DOCOL

		dw	DUPP
		dw	off_1E49
		dw	LIT
		dw	10h
		dw	SWAP
		dw	ONEP
		dw	XDO
		dw	IDO
		dw	LINE
		dw	IDO
		dw	ONE
		dw	SUBB
		dw	MOVE
		dw	XLOOP
		dw	0FFF2h
		dw	LIT
		dw	0Fh
		dw	off_1E69
		dw	SEMIS
;------------------------------------------------------------------------------
; CLEAR
		header	off_1EE5,"CLEAR",DOCOL

		dw	SCR
		dw	STORE
		dw	LIT
		dw	10h
		dw	ZERO
		dw	XDO
		dw	IDO
		dw	off_1E69
		dw	XLOOP
		dw	0FFFAh
		dw	SEMIS
;------------------------------------------------------------------------------
; MOVE
		header	off_1F04,"MOVE",DOCOL

		dw	LIT
		dw	10h
		dw	ZERO
		dw	XDO
		dw	OVER
		dw	SCR
		dw	STORE
		dw	IDO
		dw	LINE
		dw	OVER
		dw	SCR
		dw	STORE
		dw	IDO
		dw	MOVE
		dw	XLOOP
		dw	0FFEAh
		dw	DROP
		dw	DROP
		dw	5B5h
;------------------------------------------------------------------------------
; EDIT  Screen-Editor

unk_1F2C:	header	off_1F33,"EDIT",DOCOL

		dw	ZERO
		dw	LIT
		dw	6
		dw	COLOR
		dw	ZERO
		dw	LIT
		dw	28h
		dw	ZERO
		dw	LIT
		dw	20h
		dw	WINDOW
		dw	off_1E77	; L
		dw	LIT
		dw	7
		dw	PAPER
		dw	LIT
		dw	11h
		dw	EMIT
		dw	LIT
		dw	4
		dw	LIT
		dw	20h
		dw	TWO
		dw	LIT
		dw	10h
		dw	WINDOW
		dw	KEY
		dw	DUPP
		dw	DUPP
		dw	LIT
		dw	0Ch
		dw	EQUAL
		dw	ZBRAN
		dw	20h
		dw	EMIT
		dw	LIT
		dw	10h
		dw	ZERO
		dw	XDO
		dw	IDO
		dw	ZERO
		dw	LOC
		dw	IDO
		dw	off_1E5B	; T
		dw	XLOOP
		dw	0FFF4h
		dw	CR
		dw	BRAN
		dw	16h
		dw	DUPP
		dw	LIT
		dw	0Dh
		dw	EQUAL
		dw	ZBRAN
		dw	8
		dw	LIT
		dw	0Ah
		dw	EMIT
		dw	EMIT
		dw	LIT
		dw	13h
		dw	EQUAL
		dw	ZBRAN
		dw	0FFB6h
		dw	LIT
		dw	10h
		dw	ZERO
		dw	XDO
		dw	LIT
		dw	0B254h
		dw	IDO
		dw	LIT
		dw	28h
		dw	STAR
		dw	PLUS
		dw	IDO
		dw	LINE
		dw	CSLL
		dw	ZERO
		dw	XDO
		dw	TOR
		dw	DUPP
		dw	ICAT
		dw	DDUP
		dw	ZBRAN
		dw	6
		dw	BRAN
		dw	4
		dw	BL
		dw	RD
		dw	CSTOR
		dw	ONEP
		dw	FROMR
		dw	ONEP
		dw	XLOOP
		dw	0FFE2h
		dw	DROP
		dw	DROP
		dw	XLOOP
		dw	0FFC2h
		dw	ZERO
		dw	LIT
		dw	28h
		dw	ZERO
		dw	LIT
		dw	20h
		dw	WINDOW
		dw	LIT
		dw	12h
		dw	EMIT
		dw	LIT
		dw	12h
		dw	ZERO
		dw	LOC
		dw	SEMIS
;------------------------------------------------------------------------------
; DUMP
; : DUMP   CR CR BASE @ HEX ROT ROT OVER + SWAP DO I 0 4 D.R 
;          I 8 0 DO DUP I + C@ 3 .R LOOP 2 SPACES 9 EMIT
;          8 0 DO DUP I + C@ 7F AND DUP 1F > IF EMIT ELSE  ." ." DROP ENDIF 
;          LOOP DROP CR 8 +LOOP BASE ! ;

		header	DUMP,"DUMP",DOCOL,,unk_1DC2
;					wieder Vocabulary FORTH

		dw	CR
		dw	CR
		dw	BASE
		dw	AT
		dw	HEX
		dw	ROT
		dw	ROT
		dw	OVER
		dw	PLUS
		dw	SWAP
		dw	XDO
		dw	IDO
		dw	ZERO
		dw	LIT
		dw	4
		dw	DDOTR
		dw	IDO
		dw	LIT
		dw	8
		dw	ZERO
		dw	XDO
		dw	DUPP
		dw	IDO
		dw	PLUS
		dw	CAT
		dw	THREE
		dw	DOTR
		dw	XLOOP
		dw	0FFF2h
		dw	TWO
		dw	SPACS
		dw	LIT
		dw	9
		dw	EMIT
		dw	LIT
		dw	8
		dw	ZERO
		dw	XDO
		dw	DUPP
		dw	IDO
		dw	PLUS
		dw	CAT
		dw	LIT
		dw	7Fh
		dw	ANDD
		dw	DUPP
		dw	LIT
		dw	1Fh
		dw	GREAT
		dw	ZBRAN
		dw	8
		dw	EMIT
		dw	BRAN
		dw	8
		dw	PDOTQ
		db	1,'.'
		dw	DROP
		dw	XLOOP
		dw	0FFD8h
		dw	DROP
		dw	CR
		dw	LIT
		dw	8
		dw	XPLOO
		dw	0FF96h
		dw	BASE
		dw	STORE
		dw	SEMIS
;------------------------------------------------------------------------------
;TASK
TASK_:		header	TASK,"TASK",DOCOL

		dw	SEMIS
;------------------------------------------------------------------------------
INITDP:		equ	$


;------------------------------------------------------------------------------
;		MEMORY MAP
;	( THE FOLLOWING EQUATES ARE NOT REFERENCED ELSEWHERE )
;
;		LOCATION	CONTENTS
;		--------	--------
;MCOLD		EQU	ORIG	;JMP TO COLD START
;MWARM		EQU	ORIG+4	;JMP TO WARM START
;MA2		EQU	ORIG+8	;COLD START PARAMETERS
;MUP		EQU	UP	;USER VARIABLES' BASE 'REG'
;MRP		EQU	RPP	;RETURN STACK 'REGISTER'
;
;MBIP		EQU	BIP	;DEBUG SUPPORT
;MDPUSH		EQU	DPUSH	;ADDRESS INTERPRETER
;MHPUSH		EQU	HPUSH
;MNEXT		EQU	NEXT
;
;MDP0		EQU	DP0	;START FORTH DICTIONARY
;MDIO		EQU	DRIVE	;CP/M DISK INTERFACE
;MCIO		EQU	EPRINT	;CONSOLE & PRINTER INTERFACE
;MIDP		EQU	INITDP	;END INITIAL FORTH DICTIONARY
;				  = COLD (DP) VALUE
;				  = COLD (FENCE) VALUE
;				  |  NEW
;				  |  DEFINITIONS
;				  V
;
;				  ^
;				  |  DATA
;				  |  STACK
;MIS0		EQU	INITS0	;  = COLD (SP) VALUE = (S0)
;				   = (TIB)
;				  |  TERMINAL INPUT
;				  |  BUFFER
;				  V
;
;				  ^
;				  |  RETURN
;				  |  STACK
;MIR0		EQU	INITR0	;START USER VARIABLES
;				  = COLD (RP) VALUE = (R0)
;				  = (UP)
;				;END USER VARIABLES
;MFIRST		EQU	BUF1	;START DISK BUFFERS
;				  = FIRST
;MEND		EQU	EM-1	;END DISK BUFFERS
;MLIMIT		EQU	EM	;LAST MEMORY LOC USED + 1
;				  = LIMIT
;

		end
